/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2015. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Copied from RQM: com.ibm.asq.common.web.ui.internal.widgets.StationaryPane
 // Summary:
 //   A simple pane widget that is capable of holding certain contents in its content area (center).
 //   The main feature of this pane is that it maintains a constant distance from the top of the
 //   page even when the page is being scrolled down or up. This maintains the pane
 //   to be always in view. It has a built in toggle control that allows the user to turn on/off
 //   the pane's scroll tracking.
 //    
dojo.provide("rmps.ui.common.StationaryPane");

dojo.require("dijit.Tooltip");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.fx");

(function() {

var AnimateDuration = 400;

dojo.declare("rmps.ui.common.StationaryPane",
             [dijit._Widget, dijit._Templated],
{    
    templateString: '<div id="${id}" dojoAttachPoint="domNode" class="com-ibm-xtools-common-rmps-web-ui-stationary-pane">' +
                    '  <div class="content-area" dojoAttachPoint="containerNode" role="region" tabindex="-1"></div>' +
                    '</div>',
    
    title: null,
    trackMessage: null,
    untrackMessage: null,
    isDirectory: false,
    isShown: true,
    
    postCreate: function() 
    {
        this.init = false;
        if(this.title)
            this.setTitle(this.title);
            
        // create a tooltip for our tracker toggle. This is to
        // turn on/off the auto tracking of the stationary pane.
        //
        if(this.trackMessage && this.untrackMessage) {
            this.toggleTooltip = new dijit.Tooltip({label: this.untrackMessage, connectId:[this.stickyNode]});
        }
                
        // give some slack to the top of the stationary pane and the 
        // top edge of the page. The stationary pane will be below the
        // Jazz floating menu bar.
        this.slackY = 30;
        
        // initialize the tracking.
        this.track = false;
        this._changeTracking();
    },
    
    uninitialize: function(){
    	if (this.toggleTooltip){
    		this.toggleTooltip.destroyRecursive();
    		delete this.toggleTooltip;
    	}
    },
    
    // Summary:
    //    Set the title of the pane.
    //
    setTitle: function(/*String*/ title)
    {
        this.titleNode.innerHTML = title;
    },
    
    _changeTracking: function(event)
    {
        this.track = !this.track;
//        var imgName = this.track ? "unpinnedview.gif" : "pinnedview.gif";
//        this.stickyNode.src = dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/widgets/images/icons/elcl/" + imgName);
//        this.stickyNode.alt = (this.track) ? this.trackMessage : this.untrackMessage;
//        this.stickyNode.tabIndex = 0;
//        if(this.toggleTooltip) {
//            this.toggleTooltip.close();
//            this.toggleTooltip.label = (this.track) ? this.trackMessage : this.untrackMessage;
//        }
        this.lastScrollPos = 0;

        // if the tracking is turned on, add the handler to track
        // scrolling.
        //
        if(this.track) {
            this.resizeHandler = this.connect(window, "onresize", "_trackTopPosition");
            this.scrollHandler = this.connect(window, "onscroll", "_trackTopPosition");
        }
        else {
            // tracking is turned off, diable our handlers and
            // retrack back to the top.
            if(this.resizeHandler) {
                this.disconnect(this.resizeHandler);
                this.resizeHandler = null;
            }
            if(this.scrollHandler) {
                this.disconnect(this.scrollHandler);
                this.scrollHandler = null;
            }
			if (this._anim){
				this._anim.stop();
			}

            this._anim = dojo.fx.slideTo({
                node: this.domNode,
                duration: AnimateDuration,
                left: 0,
                top: 0,
				onEnd: dojo.hitch(this, "_endAnimation")
            }).play();
        }
        if(event){
            dojo.stopEvent(event);
        }
    },
    
    // Summary:
    //    Internal function to calculate the absolute distance from the top of the
    //    pane to the top of the page. This is returned as the offset from the top.
    //
    _calculateTopOffset: function()
    {
        var node = this.domNode;
        var topOffset = 0;
    
        if(node.offsetParent) {
            while(node) {
                topOffset += node.offsetTop;
                node = node.offsetParent;
            }            
        }
        else if(node.y)
            topOffset += node.y;
            
    return topOffset;
    },
    
    _trackTopPosition: function()
    {
		if( dojo.isIE && this.domNode.parentElement == null )
		{
			//Do not proceed if the domNode is not part of the document yet.
			//This circumvents problems with dojo.marginBox.
			return;
		}
    	// If the Pane is not visible (i.e. it belongs to a hidden tab) then do not track the top position
    	if( !this.isShown) {
    		return;
    	}
        var viewport = dijit.getViewport();
        var curBound = dojo.marginBox(this.domNode);
        var scrollDir = viewport.t - this.lastScrollPos;

        if(!this.init) {
            this.topOriginalOffset = this.topOffset = this._calculateTopOffset() - this.slackY;
            this.init = true;
        }
        
        if (this.track && dojo.isFunction(this.getTopOffsetSlack)) {
            this.topOffset = this.topOriginalOffset + this.getTopOffsetSlack();
        }
                
        this.lastScrollPos = viewport.t;
        
        // we have not scrolled to top of this pane
        // yet. It's still visible, don't have to move the the pane,
        // or if the panel's height is taller then the
        // viewport, don't scroll down either.
        if(scrollDir > 0 && (viewport.t < this.topOffset || (curBound.h > viewport.h))) {
            return;            
        }
        // we could be scrolling back up, make sure we don't change the top position if
        // the panel height is taller.
        var t = (curBound.h > viewport.h) ? 0 : Math.max(0, (viewport.t - this.topOffset));

        // our position is already here, don't have 
        // to move.
        if(curBound.t == t)
            return;
            
		if (this._anim){
			this._anim.stop();
		}
		
        // validate position
        // curBound.h is the widget height, t is the top in pixels for slideTo animation, 
		// topOffset is the height between the top page and this widget
        if(!this.onTrackToNewPos(curBound.h + t + this.topOffset)) {
        	return;
        }
                
        this._anim = dojo.fx.slideTo({
            node: this.domNode,
            duration: AnimateDuration,
            left: 0,
            top: t,
			onEnd: dojo.hitch(this, "_endAnimation")
        }).play();
    },
	
	_endAnimation: function()
	{ 
		this._anim = undefined;
		
		// resize
		var childWidgets = dijit.findWidgets(this.containerNode);
		dojo.forEach(childWidgets, function(w) {
			if (w.resize) {
				w.resize();
			}
		});
	},
	
    // Summary:
    // generic interface method to veto the bottom position when animate this pane 
	// when scroll or resize events.
	// override this method to validate bottom position for the widget as you need.
	// true value allows the animation to start.
    onTrackToNewPos: function(bottomPos){
    	return true;
    }
});

}());

