/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012, 2015.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.explorer.ExplorerSearchWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

//dojo.require("rmps.ui.explorer.ExplorerMain");
dojo.require("rmps.app.ui.internal.pages.design.ExploreContent");
dojo.require("rmps.ui.EllipsisLabel");
dojo.require("rmps.core.explorer.ExplorerConstants");
dojo.require("rmps.ui.explorer.ExplorerUtilities")
dojo.require("jazz.ui.SimpleToolbar");

dojo.requireLocalization("rmps.ui.explorer", "ExplorerMessages");
var messages = dojo.i18n.getLocalization("rmps.ui.explorer", "ExplorerMessages");

var ExplorerUtilities = rmps.ui.explorer.ExplorerUtilities;

dojo.declare("rmps.ui.explorer.ExplorerSearchWidget", [dijit._Widget, dijit._Templated], {

	messages: messages,
	templateString: dojo.cache("rmps.ui", "explorer/templates/ExplorerSearchWidget.html"),
	widgetsInTemplate: true,

	// attach points
	_topContainer: null,
	_titleBarNode: null,
	_menuToolbar: null,
	_explorerMain: null, //Widget

	// mixin
	projectUri: null,
	contextUri: null,
	onlyType: null,
	userUri: null,
	// ok button callback
	onOkButton: null,
	// cancel button callback
	onCancelButton: null,
	
	// props
	_titleBar: null,
		
	postCreate: function() {
		this.inherited(arguments);	
		
		this._doPostCreate();
	},
	
	_doPostCreate: function() {
		this.initializeTypes([]);
	},
	
	initializeTypes: function(projects) {
		if (this.onlyType && this.onlyType != "http://www.w3.org/2000/01/rdf-schema#Resource" && 
			this.onlyType != "http://jazz.net/ns/dm/core#Document" &&
			this.onlyType != "http://jazz.net/ns/dm/diagram#MappingState") {
			// next have to find all subclasses of onlyType
			var whereAllTypes = "?value rdfs:subClassOf* <"+ this.onlyType +">."+
			"?value rdf:type owl:Class ."+
			"?value rdf:type ?classType .";
		
			var self = this;
			// use simple select type dialog, till we have the search dialog works 
			rmps.core.client.xhrGet({
				url: net.jazz.ajax._contextRoot + "/modelvalidation/values",
				handleAs: 'json',
				headers: {
					"Accept" : "application/json",
					"X-ibm-rmps-internal" : "true"
				},
				content: {
					resourceUri: this.onlyType,
					adjustedRangeUri: this.onlyType,
					propertyUri: "http://www.w3.org/1999/02/22-rdf-syntax-ns#type",
					adjustedDomainUri: this.onlyType,
					whereClause: whereAllTypes
				},
				load: function(data, ioArgs) {
					var types = [];
					for (var i=0, lg=data.length; i<lg; i++) {
						var type = data[i].value || data[i].rdf_about;		
	                    types.push(type);
					}
					
					self.initializeExplorer(projects, types);
				}
			});
		}
		else {
			this.initializeExplorer(projects, []);
		}
	},
	
	initializeExplorer: function(projects, types) {
		this.onlyTypes = types;
		
		this.resetExplorerForProject(); // set to current project area
	},
	
	startup: function() {
		this.inherited(arguments);
		
		if (this._explorerMain) {
			this._explorerMain.startup();
		}
	},
		
	resize: function() {
		// Intentionally disabled: see WI33654
	},
		
	layout: function() {
	},
	
	onClickOkButton: function() {
		if (this.onOkButton) {
			for (var i = 0; i < this._selectedItems.length; i++) {
				var item = this._selectedItems[i];
				if (!item.rdf_about) {
					item.rdf_about = ExplorerUtilities.getItemURI(item);
				}
			}
			
			this.onOkButton(this._selectedItems);
		}
	},
	
	onClickCancelButton: function() {
		if (this.onCancelButton) {
			this.onCancelButton();
		}
	},
	
	resetExplorerForProject: function(projectUri, contextUri) {
//		if (this._explorerMain) {
//			this._explorerMain._destroyToolbar();
//			this._explorerMain.destroyRecursive();
//		}
//		
//		this._explorerMain = new rmps.ui.explorer.ExplorerMain({
//			explorerID: rmps.core.explorer.ExplorerConstants.IDs.EXPLORER_SEARCH,
//			noResourceLinks: true,
//			_projectUri: projectUri ? projectUri : this.projectUri,
//			_configurationUri: contextUri ? contextUri : this.contextUri,
//			enableMultiSelect: this.enableMultiSelect,
//			onlyTypes: this.onlyTypes,
//			resizingDisabled: true,
//		});
		
		this._explorerMain = new rmps.app.ui.internal.pages.design.ExploreContent({
			parent: this,
			projectAreaUri: projectUri ? projectUri : this.projectUri,
			configurationUri: contextUri ? contextUri : this.contextUri,
			userUri : this.userUri,
			showLoading: this.showLoading(),
			hideLoading: this.hideLoading(),
			height: "370px",
			fnOnSelect: dojo.hitch(this, this.fnOnSelect),
			multiSelect: this.enableMultiSelect,
			onViewpointChange: dojo.hitch(this, function(viewpoint) {
				this._explorerMain.setViewpointURI(viewpoint.uri);
				this._explorerMain.refresh();
			}),
			noActions: true,
			noNavigation: true,
			selectableTypes: this.onlyTypes
		});

		this._explorerContent.appendChild(this._explorerMain.domNode);
		
		// Create the tool bar
		this._explorerMain.initializeToolBar(this._menuToolbar, true /*noPinboard*/);	
		this._explorerMain.startup();
	},
	
	showLoading: function() {
		// TODO
	},
	
	hideLoading: function() {
		// TODO
	},
	
	fnOnSelect: function(items) {
		this._selectedItems = items;

		var isDisabled = dojo.hasAttr(this._okButton, "disabled");
		var hasSelection = this._selectedItems.length > 0;
		if (hasSelection && isDisabled) {
			dojo.removeAttr(this._okButton, "disabled");
		} else if (!hasSelection && !isDisabled) {
			dojo.attr(this._okButton, "disabled", "disabled");
		}
	}

});



}());
