/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2015. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

(function() {
	dojo.provide("rmps.core.oslc_config.OslcConfigClient");

	dojo.require("jazz.app.context");
	dojo.require("rmps.core.RootServices");
	dojo.require("rmps.core.util.IRmpsConstants");
	dojo.require("rmps.core.oslc_config.OslcConfig");

	var OSLC_CONFIG_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.OSLC_CONFIG, "cmServiceProviders");

	rmps.core.oslc_config.OslcConfigClient = {

			getComponentUrisFromProjectId: function(projectId) {
				var query = {
					projectId: projectId	
				};
				var args = {
					url: OSLC_CONFIG_PATH + "/configurations/queryComponent?" + dojo.objectToQuery(query),
					headers: {
						"Accept": "application/json",
						"X-ibm-rmps-internal" : "true"
					},
					handleAs: "json",
					load: function( response, ioArgs ) {
						return response;
					}// end load
				}; // end args
				return jazz.client.xhrGet(args);
			},
			
			getProjectUriFromComponentUri: function(componentUri) {
				var component = {
					componentUri: componentUri
				};
				var args = {
					url: OSLC_CONFIG_PATH + "/configurations/queryProject?" + dojo.objectToQuery(component),
					headers: {
						"X-ibm-rmps-internal": "true"
					},
					load: function(response, ioArgs) {
						return ioArgs.xhr.getResponseHeader("Location");
					}
				};
				return jazz.client.xhrHead(args);
			},
			
			getConfigurations: function(projectId, searchString, types, page) {
				// Default to the current project name is none is provided
				var query = {
					projectId : projectId ? projectId : jazz.app.context.get().itemId,
					query : searchString,
					page: page || 0
				};
				if (types) {
					query.types = types;
				}
				var args = {
					url: OSLC_CONFIG_PATH + "/configurations/query?" + dojo.objectToQuery(query),
					headers: {
						"Accept": "application/atom+json",
						"X-ibm-rmps-internal" : "true"
					},
					handleAs: "json",
					load: function( response, ioArgs ) {
						var configurations = [];		
						var entries = response.entry;
						
						configurations.totalResults = parseInt(response.totalCount[0].value, 10);
						if (entries) {
							for (var i = 0; i < entries.length; i++) {
								var oslcConfig = new rmps.core.oslc_config.OslcConfig();
								oslcConfig.parseEntry(entries[i], ioArgs);
								configurations.push(oslcConfig);
							}							
						}

						return configurations;
					}// end load
				}; // end args
				return jazz.client.xhrGet(args);
			},
			
			getRecentConfigurationsInfo: function(type, configurations, page) {
				var query = {
					type: type,
					page: page
				};
				var args = {
					url: OSLC_CONFIG_PATH + "/configurations/recentInfo?" + dojo.objectToQuery(query),
					headers: {
						"Accept": "application/atom+json",
						"Content-Type": "application/atom+json",
						"X-ibm-rmps-internal" : "true"
					},
					handleAs: "json",
					hasBody: true,
					postData: dojo.toJson(configurations),
					load: function( response, ioArgs ) {
						var configurations = [];		
						var entries = response.entry;
						
						configurations.totalResults = parseInt(response.totalCount[0].value, 10);
						if (entries) {
							for (var i = 0; i < entries.length; i++) {
								var oslcConfig = new rmps.core.oslc_config.OslcConfig();
								oslcConfig.parseEntry(entries[i], ioArgs);
								configurations.push(oslcConfig);
							}							
						}

						return configurations;
					}// end load
				}; // end args
				return jazz.client.xhrPost(args);
			}
	};
}());