/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2015.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.queryeditor.ResourceWidget");

dojo.require("dijit._Widget");

dojo.require("rmps.ui.queryeditor.QueryConstants");
dojo.require("rmps.ui.queryeditor.IQueryWidget");
dojo.require("rmps.ui.queryeditor.filters.ResourcesFilterWidget");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");

(function(){

var messages = dojo.i18n.getLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");
var QueryConstants = rmps.ui.queryeditor.QueryConstants;

dojo.declare("rmps.ui.queryeditor.ResourceWidget", [dijit._Widget, rmps.ui.queryeditor.IQueryWidget], {

	_queryConditionName: QueryConstants.RESOURCE_CONDITION_TYPE,
	_filterWidget: null,
			
	postCreate: function() {
		this.inherited(arguments);
		
		// Create value filter widget
		this._filterWidget = this.createFilter(null); // no type filtering
		if (this._filterWidget) {
			this.updateFilterData();
			this.domNode.appendChild(this._filterWidget.domNode);
		}
	},

	createFilter: function(targetType) {
		return new rmps.ui.queryeditor.filters.ResourcesFilterWidget({
			changeCallback: dojo.hitch(this, this._filterChangeCallback),
			dataType: targetType
		});
	}, 

	updateFilterData: function() {
		var conditionData = this.getConditionData();
		if (conditionData) {
			var resources = conditionData[QueryConstants.RESOURCE_CONDITION_URI_PROPERTY];
			if (resources) {
				this._filterWidget.setResources(resources);
			}
		}
	},
	
	serialize: function() {
		var obj = {};
		var condition = obj[this._queryConditionName] = {};
		condition[QueryConstants.CONDITION_PRESENTATION_PROPERTY] = this.getIdentifier();
		condition[QueryConstants.CONDITION_IGNORE_CONTEXT_PROPERTY] = false;
		
		// ResourceCondition
		condition[QueryConstants.RESOURCE_CONDITION_URI_PROPERTY] = this._filterWidget.getResources();			
		
		return obj;
	},
	
	_filterChangeCallback: function() {
		// TODO: Support removeDirty as well
		this.mainWidget.onMarkDirty("valueChange");
	},
	
	validate: function(rootTab) {
		return this.inherited(arguments); 
	}
});
}());