/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2015.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.resource.ResourceInfoClient");

dojo.require("jazz.app.context");
dojo.require("rmps.core.resource.ResourceInfo");
dojo.require("rmps.core.xhr");
dojo.require("rmps.core.RootServices");
dojo.require("rmps.core.error");
dojo.require("rmps.core.util.IRmpsConstants");

var INFO_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "info");

var OSLC_CONFIG_PARAM = rmps.core.util.IRmpsConstants.OSLC_CONFIG;
var BLANK_ICON_URL = dojo.moduleUrl("rmps.app", "graphics/icons/blank.gif").toString();

rmps.core.resource.ResourceInfoClient = {
	
	getResourceInfo: function(resourceUris, properties, configuration) {
		return this._getResourceInfos(resourceUris, properties || ['*'], configuration);
	},

	_getResourceInfos: function(resourceUris, properties, baseline) {
		var uris = resourceUris;
		if (!dojo.isArray(uris)) {
			uris = [uris];
		}
		var queryParams = {};
		var key = uris.join(',');
		if (properties) {
			if(!dojo.isArray(properties)){
				properties = [properties];
			}
			properties.sort();
			queryParams.properties = properties.join(',');
			key += queryParams.properties;
		}
		
		// Project id
		queryParams.projectId = jazz.app.context.get().itemId;
		
		// Baseline
		var baselineParam = "";
		if(!baseline && baseline !== "" ){ //quickSearch in other project contexts sets baseline to ""
			baseline = jazz.app.context.get().baseline;
		}
		if (baseline) {
			baselineParam = '&' + OSLC_CONFIG_PARAM + '=' + baseline;
		}
				
		var args = {
			url: INFO_PATH + '?' + dojo.objectToQuery(queryParams) + baselineParam,
			handleAs: "json",
			headers: {
				"Content-Type": "application/rdf+xml",
				"Accept": "application/rdf+json",
				"X-ibm-rmps-internal": "true"
			},
			hasBody: true,
			postData: this._createInfoRequest(uris),
			cacheKey: key
		};
		var d = rmps.core.xhrPost(args);
		var def = new dojo.Deferred();
		d.addBoth(function(response) {
			var descriptions = response["rdf:Description"];
			var resourceInfos = [];
			if (descriptions) {
				for (var i = 0; i < descriptions.length; ++i) {
					var resourceInfo = new rmps.core.resource.ResourceInfo();
					resourceInfo.parseDescription(descriptions[i]);
					resourceInfos.push(resourceInfo);
				}				
				if (resourceInfos.length > 0) {
					def.callback(resourceInfos);
					return;
				}
			}
			
			def.errback(rmps.core.error.createWarning("Requested resource \'" + resourceUris + "\' is not available."));
		});
		return def; 
	},
	
	_createInfoRequest: function(uris) {
		function encodeXML(text) {
			var div = dojo.doc.createElement("div");
			div.appendChild(dojo.doc.createTextNode(text));
			return div.innerHTML;
		}
		
		var result = '<?xml version="1.0" encoding="UTF-8"?>\n<rmpsInfo:elements xmlns:rmpsInfo="http://jazz.net/xmlns/rmps/info/1.0/">\n';
		for (var i = 0; i < uris.length; ++i) {
			result += '<rmpsInfo:serverUri>';
			result += encodeXML(uris[i]);
			result += '</rmpsInfo:serverUri>\n';
		}
		result += '</rmpsInfo:elements>\n';
		return result;
	},
	
	getIconUrl: function(uri, timestamp, defaultIconURL) {
		// Make sure the icon is not cached and uses the correct configuration
		var obj = {
			icon: uri,
			preventCache: timestamp ? timestamp : (new Date()).getTime(),
			defaultIcon: defaultIconURL ? defaultIconURL : BLANK_ICON_URL
		};
		var baseline = jazz.app.context.get().baseline;
		if(!baseline) {
			var qstr = window.location.search.substr(1);
			if (qstr.length) {
				var qparts = qstr.split("&");
				for (var x = 0; x < qparts.length; x++) {
					var tp = qparts[x].split("=");
					if (tp[0] === "oslc_config.context" || tp[0] === "context") {
						baseline = decodeURIComponent(tp[1]); 
					}
				}
			}
		}
		
		if (baseline) {
			obj[OSLC_CONFIG_PARAM] = baseline;
		}
		return INFO_PATH + '?' + dojo.objectToQuery(obj);
	}
};

}());