/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013, 2014, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("com.ibm.xtools.rmps.web.deliver.server.core.ui.internal.deliver.InterimResultClient");

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.CompareProvider");
var CompareProvider = com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.CompareProvider;

//var CompareConstants = com.ibm.rdm.web.compare.CompareConstants,
//ResourceType = CompareConstants.ResourceType,
//RDMClient = com.ibm.rdm.web.server.client.RDMClient,
//VvcUtils = com.ibm.rdm.web.util.vvcutils,
//VVCCompareConstants = com.ibm.rdm.web.comparevvc.VVCCompareConstants,
//INTERIM_RESULT_ERROR = "CRRRW7781";

//Set up a structure that we can iterate over to classify different resource types
var resourceTypeIndex;
function getResourceTypeIndex() {
	// summary:
	//		Get the resource type index, initialising it if that has not already been done.  Must be done lazily so that
	//		the various global values are available.
	//		For each compare provider this provides a table of prefix,type pairs and a default type to use
	//		if no match is found.
	if (!resourceTypeIndex) {
		resourceTypeIndex = {};
		resourceTypeIndex[VVCCompareConstants.COMPARE_ARTIFACT_NAMESPACE] = {
			mappings: [
				// STRUCTURE_SLUG_PREFIX must come before FRONTING_SERVICE_RESOURCES so
				// module structures are recognised before being matched by the artifact test.
				{slugPrefix: VVCCompareConstants.STRUCTURE_SLUG_PREFIX, type: ResourceType.MODULE_STRUCTURE},
				{uriPrefix: com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES, type: ResourceType.ARTIFACT},
				{uriPrefix: com.ibm.rdm.web.FRONTING_SERVICE_BOOKMARKS, type: ResourceType.BOOKMARK},
				{uriPrefix: com.ibm.rdm.web.FRONTING_SERVICE_COMMENTS, type: ResourceType.COMMENT},
				{uriPrefix: com.ibm.rdm.web.FRONTING_SERVICE_LINKS, type: ResourceType.LINK},
				{uriPrefix: com.ibm.rdm.web.FRONTING_SERVICE_TAGS, type: ResourceType.TAG},
				{uriPrefix: com.ibm.rdm.web.FRONTING_SERVICE_VIEWS, type: ResourceType.VIEW},
				{uriPrefix: com.ibm.rdm.web.FRONTING_SERVICE_GLOSSARY, type: ResourceType.GLOSSARY_TERM_INFO},
				{uriPrefix: com.ibm.rdm.web.FRONTING_SERVICE_WRAPPED_RESOURCES, type: ResourceType.WRAPPED_RESOURCE}
			],
			defaultType: ResourceType.ARTIFACT
		};
		resourceTypeIndex[VVCCompareConstants.COMPARE_FOLDER_NAMESPACE] = {
			defaultType: ResourceType.FOLDER
		};
		resourceTypeIndex[VVCCompareConstants.COMPARE_TYPE_SYSTEM_NAMESPACE] = {
		    mappings: [
	            {namePrefix: VVCCompareConstants.OBJECT_TYPE_CHANGE, type: ResourceType.OBJECT_TYPE},
	            {namePrefix: VVCCompareConstants.ATTRIBUTE_DEFINITION_CHANGE, type: ResourceType.ATTRIBUTE_DEFINITION},
	            {namePrefix: VVCCompareConstants.ATTRIBUTE_TYPE_CHANGE, type: ResourceType.ATTRIBUTE_TYPE},
	            {namePrefix: VVCCompareConstants.ENUM_ATTRIBUTE_TYPE_CHANGE, type: ResourceType.ATTRIBUTE_TYPE},
	            {namePrefix: VVCCompareConstants.LINK_CONSTRAINTS_CHANGE, type: ResourceType.LINK_CONSTRAINTS},
	            {namePrefix: VVCCompareConstants.LINK_TYPE_CHANGE, type: ResourceType.LINK_TYPE},
	            {namePrefix: VVCCompareConstants.REQ_IF_DEFINITION_CHANGE, type: ResourceType.REQ_IF_DEFINITION},
	            {namePrefix: VVCCompareConstants.REQ_IF_PACKAGE_CHANGE, type: ResourceType.REQ_IF_PACKAGE}
	        ],
			defaultType: ResourceType.TYPE
		};
	}
	return resourceTypeIndex;
}

function hasSlugPrefix(conceptURI, prefix) {
	// summary:
	//		Returns true if the slug part of the conceptURI starts with the give prefix string.
	var slugStart;
	return (slugStart = conceptURI.lastIndexOf('/') + 1) && (conceptURI.indexOf(prefix, slugStart) === slugStart);
}

function getResourceType(compareProvider, conceptURI, displayName) {
	// summary:
	//		Works out the type of an artifact based on the concept URI.  This is all a bit hacky because it looks at
	//		the form of the conceptURI.  Ideally we would get the resource type information from the server, since it already knew
	//		that when in allocated the resource to a compare provider.  However, in the absence of that, we do the
	//		sorting here - at least it's isolated to one place.
	// compareProvider:
	//		Identifier for a compare provider.  Narrows down the range of possible resourceTypes,
	//		to reduce the selection process.
	// conceptURI:
	//		URI of the resource to be checked.
	// returns:
	//		A string identifying the type of the resource.
	var compareProviderMappings = getResourceTypeIndex()[compareProvider];
	var mappings = compareProviderMappings.mappings,
		i, mappingEntry;
	if (mappings) {	
		for (var i = 0 ; i < mappings.length ; i++) {
			mappingEntry = mappings[i];
			if (
					(mappingEntry.slugPrefix && hasSlugPrefix(conceptURI, mappingEntry.slugPrefix))
				||	(mappingEntry.uriPrefix && (conceptURI.lastIndexOf(mappingEntry.uriPrefix, 0) === 0))
				||	(mappingEntry.namePrefix && displayName && (displayName.lastIndexOf(mappingEntry.namePrefix, 0) === 0))
			) {
				// got a match
				return mappingEntry.type;
			}
		}
	}
	return compareProviderMappings.defaultType;
}


function buildDiffEntries(entries, allEntriesIndex, descendants, compareProvider, isSource) {
	
	// Go through descendants, building list to output, filtering by our matching compare provider URI as we go, and
	// only including resources that have an interesting change (delta).
	descendants.forEach(function(descendantResourceDiff){
		if ((compareProvider && descendantResourceDiff.compareProvider === compareProvider) // specific compare provider 
			|| !compareProvider) { // all compare providers
			
			// this is a sanity check, at one point locks were being returned as diffs, however
			// since they have no compare provider, we don't know what to do with them.
			if (!descendantResourceDiff.compareProvider) {
				console.log("No compare provider for ", descendantResourceDiff, "...skipping");
				return;
			}
			
			var curDelta,
				// get or create the hash of entries for the particular compare provider
				currentProvider = descendantResourceDiff.compareProvider || VVCCompareConstants.COMPARE_DEFAULT_NAMESPACE,
				cpEntries = entries[currentProvider] || (entries[currentProvider] = {}),
				conceptURI = descendantResourceDiff.concept,
				// Get existing entry if there is one, otherwise create a new one.
			 	entry = allEntriesIndex[conceptURI],
			    displayName = descendantResourceDiff.displayName;
			
			if (!entry) {
				// create a new entry
				entry = cpEntries[conceptURI] = allEntriesIndex[conceptURI] = {
					concept: conceptURI,
					displayName: displayName,
					type: getResourceType(currentProvider, conceptURI, displayName)
				};
			} else if (!entry.displayName && displayName) {
			    // if the entry exists, but is missing a display name (because it was deleted in the
			    // source configuration most likely), but we have a display name from the current 
			    // descendant, then use that one.
			    entry.displayName = displayName;
			    // and now infer the "type" of the entry armed with the proper display name.
			    entry.type = getResourceType(currentProvider, conceptURI, displayName);
			}
			
			if (descendantResourceDiff.deltaType) {
			
				var descendantDeltaType = descendantResourceDiff.deltaType;
				
				// Assign the entry delta types according to the descendentDiff type
				switch(descendantDeltaType) {
					case CompareConstants.DELTA_TYPE_ADD:
						curDelta = isSource ? CompareConstants.DELTA_TYPE_SOURCE_ADD : CompareConstants.DELTA_TYPE_TARGET_ADD;
						break;
					case CompareConstants.DELTA_TYPE_DELETE:
						curDelta = isSource ? CompareConstants.DELTA_TYPE_SOURCE_DELETE : CompareConstants.DELTA_TYPE_TARGET_DELETE;
						break;
					case CompareConstants.DELTA_TYPE_CHANGE:
						curDelta = isSource ? CompareConstants.DELTA_TYPE_SOURCE_CHANGE : CompareConstants.DELTA_TYPE_TARGET_CHANGE;
						break;
					case CompareConstants.DELTA_TYPE_OTHER_CONTRIBUTOR_CHANGE:
						// There has been a change, but not in this descendant branch.  Do nothing - the real change will show up
						// as a DELTA_TYPE_CHANGE when the other descendant part of interim result is parsed.
						break;
					default:
						// shouldn't get here - but just in case there is a deltaType that we don't recognise.
						curDelta = descendantDeltaType;
						break;
				}
				com.ibm.rdm.web.comparevvc.vvccompareutils.addDeltaType(entry, curDelta);
			}
			
			// Set the source or target version, unless the received entry is a spacer, which means it does not really have a valid version.
			if (!descendantResourceDiff.spacer) {
				if (isSource) {
					entry.sourceVersion = descendantResourceDiff.version;
				} else {
					entry.targetVersion = descendantResourceDiff.version;
				}
			}
		}
	});
}

function addAncestorVersions(allEntriesIndex, ancestorDiffs) {
	// summary:
	//		Adds the ancestor version to each of the difference entries.
	// allEntriesIndex:
	//		A hash of all the entries, keyed by conceptURI.
	// ancestorDiffs:
	//		An array of ancestor entries from the interim result.
	dojo.forEach(ancestorDiffs, function(ancestor){
		var entry = allEntriesIndex[ancestor.concept];
		if (entry) {
			entry.ancestorVersion = ancestor.version;
		}
	});
}

function addMergeVersions(allEntriesIndex, mergeDiffs) {
	// summary:
	//		Adds the merge version to each of the difference entries.
	// allEntriesIndex:
	//		A hash of all the entries, keyed by conceptURI.
	// mergeDiffs:
	//		An array of ancestor entries from the interim result.
	if (!mergeDiffs) {
		return;
	}
	dojo.forEach(mergeDiffs, function(merge){
		var entry = allEntriesIndex[merge.concept];
		if (entry) {
			entry.mergeVersion = merge.version;
		}
	});
}


com.ibm.xtools.rmps.web.deliver.server.core.ui.internal.deliver.InterimResultClient = {

	loadInterimResult: function(compareProvider /*?URI String*/, interimResultURI) {
		// summary:
		//		Load the interim results from VVC.
		// compareProvider:
		//		Optional.  If provided, only entries that match this compare provider namespace
		//		are returned. 
		// returns:
		//		A promise that resolves with parsed interim results as returned by parseInterimResult.
		if (!interimResultURI) {
		    interimResultURI = VvcUtils.getRequestParmValue('interimResultUri');
		    interimResultURI = decodeURIComponent(interimResultURI);
		}
		
		/*
		 * WI: 94158
		 * Each step of the compare UI, (Types, Folders, Artifacts) is currently displayed
		 * in it's own IFRAME (an ExternalWizardStep). As a result of that, each step requests
		 * the Interim Results from the server when it starts up.
		 * 
		 * The DeliverWizardApplication creates an instance it exposes globally as InterimResultClient
		 * we can use that one which will cache requests to the same /deliver/<session-id> in 
		 * the RDMCache that it will use.
		 */
		if (window.parent !== window && typeof window.parent.InterimResultClient === "object") {
		    return window.parent.InterimResultClient.loadInterimResult(compareProvider, interimResultURI);
		}
		return jazz.client.xhrGet({
			uri: interimResultURI,
			handleAs: "json",
			headers: {
				"Accept": "application/json"
			}
		})
		.then(dojo.hitch(this, this.parseInterimResult, compareProvider));
	},
	
	parseInterimResult: function(compareProvider /*?URI String*/, interimResult) {
		// summary:
		//		Parse the interim results, converting them to info on from and to versions,
		//		and (optionally) filtering by compare provider.  The received interim result
		//		can contain either 2-way or 3-way comparison information, depending on how the user
		//		selected configurations to compare.
		//		For the 3-way case we have info on changes between an ancestor and each of two
		//		descendant configurations.  What we really want it a comparison between the two
		//		descendants, so we condense the two sets of comparison information into a single
		//		set that describes the difference between the two descendant configurations.
		//		To achieve this, the Delta Type for one of the differences is flipped around making
		//		and add a delete and vice versa.  A change delta in either descendant (or both)
		// 		consitutes a change delta for the overall result.
		// interimResult:
		//		Interim result as received from VVC.
		// compareProvider:
		//		Optional.  If provided, only entries that match this compare provider namespace
		//		are returned. 
		// returns:
		//		An object of the form {
		//			targetConfigurationURI: URI string,
		//			targetConfigurationName: String,
		//			sourceConfigurationURI: URI string,
		//			sourceConfigurationName: String,
		//			operation: {},	An object with a single field.  The field name is one of isCompare, isDeliver
		//							or isAccept, and the value is true.  This makes it easy to test for the type
		//							of the current operation by code like:
		//							if (operation.isAccept) ....
		//			entries: {}		A hash keyed by compare provider namespace URI.  If compareProvider
		//							parameter is supplied, there is just one entry, matching that compareProvider.
		//							If compareProvider parameter is not supplied, all entries are returned
		//							grouped under their compare providers.
		//							Each value is an hash, keyed by conceptURI, each value being an object like:
		//								{
		//									concept: uri,
		//									deltaTypes: (Optional): A hash with keys being delta type constants and values all being true
		//									sourceVersion:	URI for the source version of the resource
		//									targetVersion:	URI for the target version of the resource
		//									ancestorVersion:	URI for the common ancestor version of the resource
		//									mergeVersion:	URI for the version that VVC will used for the merge if we don't tell it otherwise.  Only populated for merge.
		//									type:			String indicating the ResourceType of the resource.
		//								}
		//							Additional fields and deltaType values are added to these starting values as relationships are
		//							established between delta entries later in the execution.  A "related" field holds values to identify
		//							resources and links that are connected in some way with a resource - used to connect resources that
		//							need to be delivered or not delivered together.
		
		if (!interimResult.doc /*|| !interimResult.doc.ancestor*/ || !interimResult.doc.descendant) {
			throw new Error(INTERIM_RESULT_ERROR);
		}
		
		var interimResultDoc = interimResult.doc,
			operation = {},
			sessionId,
			sourceConfiguration, targetConfiguration;
		// Set one flag in the operation object.
		// Temporary way to determine operation type.  Need to also add a way to distinguish accept operation.
		if (interimResult.doc.merge) {
			operation.isDeliver = true;
		} else {
			operation.isCompare = true;
		}
		
		sourceConfiguration = interimResultDoc.descendant;
		targetConfiguration = interimResultDoc.otherDescendant;
		sessionId = interimResultDoc.sessionUUID;

		var entries = {},			// hash, key is compare provider.  each value is hash of entries for that compare provider. 
			allEntriesIndex = {},	// hash of all entries that are anywhere in "entries"
			parsedResult = {
				targetConfigurationURI: targetConfiguration.contributorUri,
				targetConfigurationName: targetConfiguration.contributorDisplayName,
				sourceConfigurationURI: sourceConfiguration.contributorUri,
				sourceConfigurationName: sourceConfiguration.contributorDisplayName,
				operation: operation,
				entries: entries,
				sessionId: sessionId
			};
		
		if (compareProvider) {
			// Make sure we return at least an empty hash for the requested compare provider.
			entries[compareProvider] = {};
		}
		
		buildDiffEntries(entries, allEntriesIndex, sourceConfiguration.resourceDiffs, compareProvider, true/*isSource*/);
		buildDiffEntries(entries, allEntriesIndex, targetConfiguration.resourceDiffs, compareProvider, false/*isSource*/);
		addAncestorVersions(allEntriesIndex, interimResultDoc.ancestor.resourceDiffs);
		// Add merge versions if there are any
		addMergeVersions(allEntriesIndex, interimResultDoc.merge && interimResultDoc.merge.resourceDiffs);
		return parsedResult;
	},
	
	getCompareProviders: function(sessionUri) {
		var args = {
			url: sessionUri + "/compareProviders",
			headers: {
				"Accept": "application/json",
				"X-ibm-rmps-internal" : "true"
			},
			handleAs: "json",
			load: function( response, ioArgs ) {
				var list = [];
				for (var key in response) {
					if (response[key]["http://www.w3.org/1999/02/22-rdf-syntax-ns#type"] &&
							response[key]["http://www.w3.org/1999/02/22-rdf-syntax-ns#type"][0].value === "http://jazz.net/ns/vvc#CompareProviderList") {
						var cps = response[key]["http://jazz.net/ns/vvc#compareProvider"];
						if (cps) {
							for (var j=0; j<cps.length; j++) {
								var cp = response[cps[j].value];
								if (cp) {
									list.push(new CompareProvider(cp));
								}
							}
						}
						break;
					}
				}
				return list;
			}// end load
		}; // end args
		return jazz.client.xhrGet(args);
	}
};

}());
