/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013, 2014, 2015.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){

dojo.provide("com.ibm.xtools.rmps.web.deliver.server.core.ui.internal.deliver.SourceTargetPickerStep");

dojo.require("dijit.form.Button");

dojo.require("jazz.ui.StyledBox");
dojo.require('jazz.ui.Select');
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("com.ibm.team.repository.web.configurationManagement.cache.CmCacheFactory");
dojo.require("com.ibm.team.repository.web.configurationManagement.uiNodes.ConfigurationUiNode");

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.Constants");
dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.ChangesetUiNode");

dojo.requireLocalization("com.ibm.xtools.rmps.web.deliver.server.core.ui.internal.deliver", "SourceTargetPickerStepMessages");



var CmCacheFactory = com.ibm.team.repository.web.configurationManagement.cache.CmCacheFactory;
var ChangesetUiNode = com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.ChangesetUiNode;
var ConfigurationUiNode = com.ibm.team.repository.web.configurationManagement.uiNodes.ConfigurationUiNode;
var WizardStep = jazz.ui.wizard.WizardStep;
var Constants = com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.Constants;
/**
 * Wizard step that allows the user pick source and target configurations to deliver/comapre/rebase.
 */
dojo.declare("com.ibm.xtools.rmps.web.deliver.server.core.ui.internal.deliver.SourceTargetPickerStep", [WizardStep, dijit._Widget, dijit._Templated], {
	
	messages : dojo.i18n.getLocalization("com.ibm.xtools.rmps.web.deliver.server.core.ui.internal.deliver", "SourceTargetPickerStepMessages"),	
	templateString: dojo.cache('com.ibm.xtools.rmps.web.deliver.server.core', 'ui/internal/deliver/templates/SourceTargetPickerStep.html'),
	widgetsInTemplate: true,
	
	/**
	 * The session
	 */
	session: null,
	
	/**
	 * The DeliverWizardApplication instance
	 */
	wizardApp: null,
	
	/**
	 * ConfigurationResource 
	 */
	source: null,
	
	/**
	 * ConfigurationResource
	 */
	target: null,
	
	/**
	 * Source picker
	 */
	sourcePicker: null,
	
	/**
	 * Target picker
	 */
	targetPicker: null,
	
	/**
	 * Space picker
	 */
	spacePicker: null,
	
	/**
	 * Spaces?
	 */
	spaces: null,
	
	/**
	 * URI of space. Automatically determined
	 */
	spaceUri: null,
	
	_cache: null,
	
	_spaceUriInit: false,
	
	canProceed: true,
	
	/**
	 * Arguments:
	 * - session: DeliverSession.js object
	 */
	constructor: function(args){
		this._cache = CmCacheFactory.getConfigurationCache();
		if (args) {
			if (args.session) {
				this.session = args.session;
			}
			if (args.wizardApp) {
				this.wizardApp = args.wizardApp;
			}
		}
		
		this.stepTitle = this.messages.step1Title;
	},
	
	postCreate: function(){

		if (this.session.isCompareOnly()) {
			this.description.innerHTML = this.messages["desc_compare"];
		} else {
			this.description.innerHTML = this.messages["desc_deliver"];
		}
		
		var sourceUri = this.session.getSourceUri();
		if (sourceUri) {
			dojo.when(this._cache.get(sourceUri), dojo.hitch(this, this.setSource));
		}
		
		var targetUri = this.session.getTargetUri();
		if (targetUri) {
			dojo.when(this._cache.get(targetUri), dojo.hitch(this, this.setTarget));
		}
		
		if (this.session.isCompareOnly()) {
			this.description.innerHTML = this.messages.desc_compare;
		} else if (this.session.isRebase()) {
			this.description.innerHTML = this.messages.desc_rebase;
		}
	},

	swap: function() {
		var temp = this.source;
		this.setSource(this.target);
		this.setTarget(temp);
	},
	
	disableUi: function() {
		if (this.sourcePicker) {
			this.sourcePicker.disabled = true;
		}
		
		if (this.targetPicker) {
			this.targetPicker.disabled = true;
		}
		
		if (this.spacePicker) {
			this.spacePicker.disabled = true;
		}
		
	},
	
	setSource: function(/*ConfigurationResource*/ config){
		this.source = config;
		dojo.removeClass(this.sourceSelectionIconNode);
		dojo.empty(this.sourceSelectionNode);
		if (config) {
			var uiNode;
			if(config.type === "changeset"){
				uiNode = new ChangesetUiNode({
					changeset: config,
					includeLink: false,
					includeHover: false
				});
			} else {
				uiNode = new ConfigurationUiNode({
					uri: config.id,//config.uri,
					includeLink: false
				});
			}
			this.sourceSelectionNode.appendChild(uiNode.domNode);
		}		
		this.validateNext();
	},
	
	setTarget: function(/*ConfigurationResource*/ config){
		this.target = config;
		dojo.removeClass(this.sourceSelectionIconNode);
		dojo.empty(this.targetSelectionNode);
		if (config) {
			var uiNode;
			if(config.type === "changeset"){
				uiNode = new ChangesetUiNode({
					changeset: config,
					includeLink: false,
					includeHover: false
				});
			} else {
				uiNode = new ConfigurationUiNode({
					uri: config.id,//config.uri,
					includeLink: false
				});
			}
			this.targetSelectionNode.appendChild(uiNode.domNode);
		}		
		this.validateNext();
	},
	
	canFinish: function() {
		if (this.session) {
			return this.session.isCompareOnly();
		} else {
			return false;
		}
	},
	
	onFinish: function() {
		this.wizardApp.performFinish();
	},
	
	validateNext: function() {
		var validate = this.validate();
		if (validate.length > 0) {
			this._showValidationWarning(validate[0]);
		} else {
			this._showValidationWarning("");
		}
		
		this.triggerButtonStateUpdate();	
		
		// finally, disable all of UI if session is no longer in uninitialized
		// mode or is not available entirely
		if (!this.session
				|| this.session.getState() !== Constants.DELIVER_SESSION_STATE_UNINITIALIZED) {
			this.disableUi();
		}
	},
	
	canFlipToNextPage: function __canFlipToNextPage() {
		var isLastStep = this.wizard && this.wizard.validSteps[this.wizard.validSteps.length-1] <= this.wizard.selectedStepPosition;
		if (!this.canProceed) {
			return false;
		} else {
			return !isLastStep && this.inherited(arguments);
		}
		
	},
	
	validate: function() {

		if (this.source == null || this.target == null) {
			return [this.messages.noSourceTargetSelected];
		} else if (this.source.id === this.target.id) { // [SW] - was .uri
			return [this.messages.cannotCompareToSelf];
		} else {
			return [];
		}
	},
	
	_showValidationWarning: function(message) {
		this.validationWarningLabel.innerHTML = message;
	},
	
	focusStep: function(){
	}
});

})();