/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.xtools.rmps.web.deliver.shared.web.ui.context.ConfigurationContextBannerButton');

dojo.require("jazz.ui.ScrollableMenu");
dojo.require('com.ibm.xtools.rmps.web.deliver.shared.web.ui.widgets.HideableMenuItem');
dojo.require('com.ibm.xtools.rmps.web.deliver.shared.web.ui.picker.WorkspacePickerDialog');
dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.context.ContextUtils");

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.changesets.CreateChangesetForm");
dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.configurations.AddToGlobalConfigurationForm");
dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.configurations.CreateConfigurationForm");

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.ConfigurationResource");
dojo.require("com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.GlobalConfigurationResource");
dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.configurations.GlobalConfigurationUtils");

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.changesets.ChangesetsClient");
dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.UIUtils");
dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.LinkToWorkItemUtil");
dojo.require("com.ibm.xtools.rmps.web.deliver.shared.oslc.web.utils.CLMUtils");
var CLMUtils = com.ibm.xtools.rmps.web.deliver.shared.oslc.web.utils.CLMUtils;

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.SecurityServiceUtils");
var SecurityUtils = com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.SecurityServiceUtils;

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.SecurityConstants");
var SecurityConstants = com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.SecurityConstants;

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.VvcUtils;

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.InfoContent");
dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.InfoContentUtils");
var InfoContentUtils = com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.InfoContentUtils;

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.configurations.GlobalConfigurationUtils");
var GlobalConfigurationUtils = com.ibm.xtools.rmps.web.deliver.shared.web.ui.configurations.GlobalConfigurationUtils;

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.ChangesetResource");
var ChangesetResource = com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.ChangesetResource;

dojo.require("jazz.util.html");
dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.VvcConstants");
dojo.require('jazz.ui.Dialog');
dojo.require('jazz.ui.MessageBox');

dojo.requireLocalization("com.ibm.xtools.rmps.web.deliver.shared.web.ui.context", "ConfigurationContextMessages");

(function ()
{
	var VvcConstants = com.ibm.xtools.rmps.web.deliver.shared.web.ui.VvcConstants;
	var UIUtils = com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.UIUtils;
	var ContextUtils = 	com.ibm.xtools.rmps.web.deliver.shared.web.ui.context.ContextUtils;
	var ChangesetsClient = com.ibm.xtools.rmps.web.deliver.shared.web.ui.changesets.ChangesetsClient;
	
	var DCTERMS_TITLE = "http://purl.org/dc/terms/title";

	var areGcsEnabled = true; // enable by default
	// GlobalConfigurationUtils.areGlobalConfigurationsEnabled(true).then(dojo.hitch(this, function(enabled){
	//	areGcsEnabled = enabled;
	// }));
	
	/**
	 * ConfigurationContextBannerButton is a configuration context combo box to be placed in the Jazz UI bannner.
	 * It allows the user to choose a configuration context, and remembers the last 5 used. It also gives entry points
	 * into the VVC application by allowing the user to go the main VVC configuration explorer page or create workspaces
	 * or snapshots.
	 * 
	 * Pass "workspaceBasedOffSnapshotOnly" optional property to the first arguments object in the constructor to only 
	 * allow workspace creation off of a snapshot.
	 */
	dojo.declare('com.ibm.xtools.rmps.web.deliver.shared.web.ui.context.ConfigurationContextBannerButton', [dijit._Widget, dijit._Templated], 
		{
			messages: dojo.i18n.getLocalization("com.ibm.xtools.rmps.web.deliver.shared.web.ui.context", "ConfigurationContextMessages"),
			templatePath: dojo.moduleUrl('com.ibm.xtools.rmps.web.deliver', 'shared/web/ui/context/templates/ConfigurationContextBannerButton.html'),
			
			/**
			 * The index of the selected item, or the item itself.
			 * @field
			 * @type Number|Widget
			 */
			selected: null,
			redirectUriSelector: null,
			onContextChange: null,
			authenticated: false,
			authenticationDeferred: null,
			menuSectionMap: null,
			
			constructor: function __ConfigurationContextBannerButton(args)
			{
				menuSectionMap = {};
				if (args)
				{
					if (args.workspaceBasedOffSnapshotOnly)
						this._workspaceBasedOffSnapshotOnly = args.workspaceBasedOffSnapshotOnly;
					
					if (args.redirectUriSelector)
						this.redirectUriSelector = args.redirectUriSelector;
				}	
			},

			postCreate: function __postCreate() {
				this._initializing = true;
				var deferreds = [this._initializeChangesetSubscribers(),
				this._initializeMenuButton(), this._initializeRecentlyUsed(),
				this._initializeMoreMenuItem(), this._initializeQuickCreateMenuItem(),
				this._initializeSelectedMenuItem(), this._initializeManageConfigurations(),
				this._initializeGlobalConfigurations(), this._initializeManageChangesets()];
				var dl = new dojo.DeferredList(deferreds);
				dl.then(dojo.hitch(this, function __finishPostCreate(result) {
					this._initializing = false;
					this._toggleMenuItems();
					//Listen to update the widget when the context is completely initialized.
					this.subscribe("vvc.context_initialized",this._initializeSelectedMenuItem);
				}));
				this._toggleMenuItems();
			},
			
			_initializeChangesetSubscribers: function __initializeChangesetSubscribers()
			{
				var initChangesetSubscribersFinished = new dojo.Deferred();
				dojo.subscribe("vvc.changeset_created", dojo.hitch(this, function() 
				{
					var changeset = arguments[0];
					if (this.redirectUriSelector != null && changeset != null && changeset.configurationUri != null && this.onContextChange != null)
					{
						var csConfiguration = new com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.ConfigurationResource({uri : changeset.configurationUri}); //not required to load yet
						//expanded context should be null for changesets, at least until they are better integrated with GCs.
						this.onContextChange(csConfiguration, null);
					}
				} ));
				initChangesetSubscribersFinished.resolve(true);
				return initChangesetSubscribersFinished;
			},
			
			_checkValid: function __checkValid(){
				var configuration = ContextUtils.getCurrentContext(true);
				if (!configuration) {
					InfoContentUtils.showSimpleContentDialog(this.messages.archivedTitle, this.messages.archivedUnknownMessage);
					return false;
				}
				if (configuration.isArchived()) {
					InfoContentUtils.showSimpleContentDialog(this.messages.archivedTitle, dojo.string.substitute(this.messages.archivedMessage, [configuration.getTitle()]));
					return false;
				}
				
				return true;
			},
			
			addChild: function __addChild(widget, insertIndex){
				//add it to the menu
				this.connect(widget, "onClick", dojo.hitch(this, this._wasClicked, widget));
				this._menu.addChild.apply(this._menu, arguments);
				
				if (widget.configuration == null)
					return;
				
				//we have both parameters to distinguish between configurations with and without changesets
				ContextUtils.addConfigurationToRecentlyUsed(widget.configuration, widget.expandedContext);
				
				var context = {
					self: this,
					recentConfig: widget.configuration,
					recentExpandedConfig: widget.expandedContext,
					configurationPopup: null
				};

				var configurationPopup = new jazz.ui.HoverPopup({
					around: widget.domNode,
					html: this.messages.loadingConfigurationPopup,
					openDelay: 300,
					maxWidth: 800,
					onOpen: dojo.hitch(context, function() {
						if (!this.self.authenticated) {
							if (!this.self.authenticationDeferred) {
								this.self.authenticationDeferred = new dojo.Deferred();
							}
							CLMUtils.authenticate(this.self.authenticationDeferred);
							this.self.authenticationDeferred.addCallback(this, dojo.hitch(this, function(result){
								this.self.authenticated = true;
								if (result) {
									for (var i = 0; i < result.length; i++) {
										if (result[i].length >= 2) {
											if (result[i][1]) {
												context.self.providers = result[i][1];
												break;
											}
										}
									}
								}
								this.self._makePopupContents(this.configurationPopup, this.recentConfig, this.recentExpandedConfig);
							}));
						} else {
							this.self._makePopupContents(this.configurationPopup, this.recentConfig, this.recentExpandedConfig);
						}
					})
				});
				
				context.configurationPopup = configurationPopup;
				
				//remove any menuItems that are no longer in the recently used list
				var menuItems = this._menu.getChildren();
				for(var i in menuItems) {
					var menuItem = menuItems[i];
					var configuration = menuItem.configuration;
					if (configuration && !ContextUtils.isConfigurationInRecentlyUsed(configuration, menuItem.expandedContext))
						this._menu.removeChild(menuItem);
				}
			},
			
			_initializeMenuButton: function __initializeMenuButton()
			{
				var initMenuButtonFinished = new dojo.Deferred();
				this._menu = new jazz.ui.ScrollableMenu({
					selectable: true,
					onOpen: dojo.hitch(this, this._onMenuOpen),
					onClose: dojo.hitch(this, this._onMenuClose)
				});
				this._menu.addChild(new jazz.ui.menu.NodeMenuItem({isTitle: true, label: this.messages.menuTitle}));
				
				this._menuPopup = new jazz.ui.MenuPopup({group: "jazz-app-BannerButtons", align: "right", menuProvider: dojo.hitch(this, this._menuProvider), menuClass:"jazz-app-Navbar-menu", injectCaret: true}, this._button);
				initMenuButtonFinished.resolve(true);
				return initMenuButtonFinished;
			},
			
			_menuProvider:function __menuProvider() {
				return this._menu;
			},
			
			_isValid:function __isValid(configuration) {
				if (!configuration){
					return true;
				}

				var uri = configuration.uri;
				var isGCURL = com.ibm.xtools.rmps.web.deliver.shared.web.ui.configurations.GlobalConfigurationUtils.isGlobalConfigurationUrl(uri);
				var config = isGCURL ? new com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.GlobalConfigurationResource() : new com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.ConfigurationResource();
				var uriget = VvcUtils.getVvcUri() + "/repository/configuration?uri="+uri;
				config.load(uriget);
				
				// This checks to see if the change set is shared and provides user with the message that they cannot select a shared change set.
				if (!config.uri ) {
					InfoContentUtils.showSimpleContentDialog(this.messages.archivedTitle, this.messages.sharedMessage);
					 return false;
				}

				else if (!isGCURL && (config.isWorkspace() || config.isSnapshot())) {
					if (config.isArchived()) {
						 InfoContentUtils.showSimpleContentDialog(this.messages.archivedTitle, this.messages.archivedUnknownMessage);
						return false;
					}
				}					
				
				return true;
			},
			_setSelectedAttr: function __setSelectedAttr(item) {
				if (typeof item == "number") {
					var widgets = this._menu.getChildren();
					if (item < widgets.length) {
						item = widgets[item];
					}
				}
				if (this.selected == item || !item.isConfiguration) return;
				
				// if archived workspace or completed change set, provide error to user
				if (item.configuration != null) {
					if (!this._isValid(item.configuration)) {
						return;
					}
				}
				
				if (item != null && item.domNode) {
					this._menu&&this._menu.setSelected(item);			
					
					if (item.iconClass) {
						if (this._previousIconClass){
							dojo.removeClass(this._icon, this._previousIconClass);
						}
						dojo.addClass(this._icon, item.iconClass);
						this._previousIconClass = item.iconClass;
					} else {
						dojo.empty(this._icon);
					}
					this._label.innerHTML = UIUtils.truncateString(item.label);
				} 
				
				if (typeof this.selected !== "number" && this._initializing == false) this.onChange(item);
				this.selected = item;
			},

			_wasClicked: function __wasClicked(item) {
				this.set("selected", item);
			},

			_initializeSelectedMenuItem: function __initializeSelectedMenuItem()
			{
				var initSelectedMenuItemFinished = new dojo.Deferred();
				var currentContext = ContextUtils.getCurrentContext();
				var expandedContext = ContextUtils.getExpandedContext();
				
				this._button.title = this.messages.contextTitle; 
				if (currentContext == null) {
					selectedMenuItem = new com.ibm.xtools.rmps.web.deliver.shared.web.ui.widgets.HideableMenuItem({
						label : this.messages.pleaseSelect,
						"class" : "com-ibm-team-vvc-context-menuItem"
					});
				} else {
					selectedMenuItem = this._getMenuItemForConfiguration(currentContext, expandedContext);
					if (selectedMenuItem == null)
					{
						selectedMenuItem = this._constructMenuItem(currentContext, expandedContext);
						//43291 - If the currect context is archived, It should not be added to Recently Used List
						if (!currentContext.isArchived())
						  this.addChild(selectedMenuItem, 1);
					}
					if (expandedContext) {
						this._button.title += "\n" + dojo.string.substitute(this.messages.currentContext, { currentContextTitle: currentContext.getTitle() }) + "\n" + dojo.string.substitute(this.messages.expandedContext, { expandedContextTitle: expandedContext.getTitle() });
					}
				}
				this._setSelectedAttr(selectedMenuItem);
				initSelectedMenuItemFinished.resolve(true);
				return initSelectedMenuItemFinished;
			},
			
			_getMenuItemForConfiguration: function __getMenuItemForConfiguration(configuration, expandedConfiguration)
			{
				var menuItems = this._menu.getChildren();
				for(var i in menuItems) {
					var menuItem = menuItems[i];
					var menuConfig = menuItem.configuration;
					var menuExpandedConfig = menuItem.expandedContext;
					if (menuConfig && configuration.uri == menuConfig.uri) {
						//we need to consider both configurations to distinguish
						//between configurations with and without changesets
						if (!expandedConfiguration) {
							if (!menuExpandedConfig) {
								return menuItem;
							}
						} else if (menuExpandedConfig && (expandedConfiguration.uri === menuExpandedConfig.uri)) {
							return menuItem;
						}
					}
				}
				return null;
			},
			
			_onMenuOpen: function __onMenuOpen() {
				this._open = true;
				this._toggleActiveClass(true);
				this._menu.domNode.style.width = "";
				var menuWidth = dojo.position(this._menu.domNode).w;
				var thisWidth = dojo.position(this.domNode).w;
				if (menuWidth < thisWidth) {
					dojo.style(this._menu.domNode, 'width', thisWidth + 'px');
				}
				if (this.selected) {
					this._menu.setSelected(this.selected);
				}
			},

			_onMenuClose: function __onMenuClose() {
				this._open = false;
				this._toggleActiveClass(false);
			},
			
			_isRecentlyUsedEnabled: function __isRecentlyUsedEnabled() {
				var deferred = new dojo.Deferred();
				deferred.resolve(true);
				return deferred;
			},
			
			_initializeRecentlyUsed: function __initializeRecentlyUsed()
			{
				var initRecentlyUsedFinished = new dojo.Deferred();
				this._isRecentlyUsedEnabled().then(dojo.hitch(this, function __initRecentlyUsedMenuItem(enabled){
					if (!enabled) { return; }
					var recentlyUsed = ContextUtils.getRecentlyUsedConfigurations();
					
					for(var i in recentlyUsed) {
						var recentConfig = recentlyUsed[i];
						if(areGcsEnabled || !recentConfig.expandedContext){ 
							var menuItem = this._constructMenuItem(recentConfig.currentContext, recentConfig.expandedContext);
							this.addChild(menuItem, 1);
						}
					}
					initRecentlyUsedFinished.resolve(true);
				}));
				return initRecentlyUsedFinished;
			},
			
			isWorkspaceCreateEnabled: function __isWorkspaceCreateEnabled() {
				var deferred = new dojo.Deferred();
				deferred.resolve(true);
				return deferred;
			},
			
			isSnapshotCreateEnabled: function __isSnapshotCreateEnabled() {
				var deferred = new dojo.Deferred();
				deferred.resolve(true);
				return deferred;
			},
			
			_initalizeCreateWorkspaceMenuItem: function __initializeCreateWorkspaceMenuItem() {
				// Can never create a workspace off a change set.
				var currentContext = ContextUtils.getCurrentContext();
				if(currentContext != null && currentContext.isChangeset()) {
					return;
				}
				
				var self = this;
				this._addMenuSectionIfNeeded(this.messages.quickCreateMenuTitle);
				this._newWorkspaceMenuItem = new com.ibm.xtools.rmps.web.deliver.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.newWorkspace,
					iconClass : "com-ibm-team-vvc-context-icon-new-workspace",
					onClick: function() {
						if (self._checkValid()) {
							self._newWorkspace();
						}
					},
					isInvisible: dojo.hitch(this, function(context) {
						if (this._workspaceBasedOffSnapshotOnly)
						{
							if (context.isSnapshot())
								return true;
							else
								return false;
						}
						return true;
					}),
					
					getDisabledText: dojo.hitch(this, function(context){
						if (!context.permissions || dojo.indexOf(context.permissions, SecurityConstants.permissions.createOrModifyWorkspace) < 0) {
							return SecurityUtils.getNoPermissionMessage(context.getProperty("controllingAreaName"));
						}
						return null;
					})
				});		
				this.addChild(this._newWorkspaceMenuItem);
			},
			
			_initializeCreateSnapshotMenuItem: function __initializeCreateSnapshotMenuItem(){
				var self = this;
				this._addMenuSectionIfNeeded(this.messages.quickCreateMenuTitle);
				this._newSnapshotMenuItem = new com.ibm.xtools.rmps.web.deliver.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.newSnapshot,
					iconClass : "com-ibm-team-vvc-context-icon-new-snapshot",
					onClick: function() {
						if (self._checkValid()) {
							self._newSnapshot();
						}
					},
					isInvisible: function(context) {
						if (context.isWorkspace())
							return true;
						else
							return false;
					},
					
					getDisabledText: dojo.hitch(this, function(context){
						if (!context.permissions || dojo.indexOf(context.permissions, SecurityConstants.permissions.createOrModifySnapshot) < 0) {
							return SecurityUtils.getNoPermissionMessage(context.getProperty("controllingAreaName"));							
						}
						return null;
					})
				});				
				this.addChild(this._newSnapshotMenuItem);
			},
			
			_initializeQuickCreateMenuItem: function __initializeQuickCreateMenuItem(){
				var initQuickCreateMenuItemFinished = new dojo.Deferred();
				var dl = new dojo.DeferredList([this.isWorkspaceCreateEnabled(), this.isSnapshotCreateEnabled()]);
				dl.then(dojo.hitch(this, function __initQuickCreateMenuItem(res) {
					if(res[0][0] && res[0][1]) { //No errors and enabled is true
						this._initalizeCreateWorkspaceMenuItem();
					}
					if(res[1][0] && res[1][1]) { //No errors and enabled is true
						this._initializeCreateSnapshotMenuItem();
					}
					initQuickCreateMenuItemFinished.resolve(true);
				}));
				return initQuickCreateMenuItemFinished;
			},
			
			isMoreMenuItemEnabled: function __isMoreMenuItemEnabled() {
				var deferred = new dojo.Deferred();
				deferred.resolve(true);
				return deferred;
			},
			
			_initializeMoreMenuItem: function __initializeMoreMenuItem()
			{
				var initMoreMenuFinished = new dojo.Deferred();
				var self = this;
				this.isMoreMenuItemEnabled().then(dojo.hitch(this, function __initMoreMenuItem(enabled) {
					if (!enabled) { return; }
					var moreMenuItem = new com.ibm.xtools.rmps.web.deliver.shared.web.ui.widgets.HideableMenuItem({
						label : this.messages.more,
						onClick: function() {
							var args = {};
							args.spaceUri = ContextUtils.getSpace();
							var picker = new com.ibm.xtools.rmps.web.deliver.shared.web.ui.picker.WorkspacePickerDialog(args);
							picker.onOk = function(configuration) {
								if (!self._isValid(configuration))							
									return;
								ContextUtils.setCurrentContext(configuration).then(dojo.hitch(this, function __currentContextSet() {
									self._initializeSelectedMenuItem();
									
									if (self.onContextChange) {
										self.onContextChange(configuration);
									}
								}),
								function __currentContextFailedInInitMoreMenuItem(err) {
									self.displayPopup(self.messages.failedToObtainGcContributions);
								});
							};
						}
					});				
					this.addChild(moreMenuItem);
					initMoreMenuFinished.resolve(true);
				}));
				return initMoreMenuFinished;
			},
			
			
			_initializeCreateChangesetsMenuItem: function __initializeCreateChangesetsMenuItem() {
				var self = this;
				this._addMenuSectionIfNeeded(this.messages.manageChangesets);
				
				this._createChangesetsMenuItem = new com.ibm.xtools.rmps.web.deliver.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.createChangeset,
					iconClass : "com-ibm-team-vvc-context-icon-create-changeset",
					onClick: function() {
						if (self._checkValid()) {
							var form = new com.ibm.xtools.rmps.web.deliver.shared.web.ui.changesets.CreateChangesetForm({
								"configuration": ContextUtils.getCurrentContext().uri
							});
							form.show();
						}
					},
					isInvisible: function(context) {
						if (context != null && context.isWorkspace())
							return true;
						return false;
					},
					getDisabledText: dojo.hitch(this, function(context){
						if (!context.permissions || dojo.indexOf(context.permissions, SecurityConstants.permissions.update) < 0) {
							return SecurityUtils.getNoPermissionMessage(context.getProperty("controllingAreaName"));
						}
						return null;
					})
					
				});				
				this.addChild(this._createChangesetsMenuItem);
			},
			
			_initializeRenameChangesetMenuItem: function __initializeRenameChangesetMenuItem() {
				var self = this;
				this._addMenuSectionIfNeeded(this.messages.manageChangesets);
				this._renameChangesetMenuItem = new com.ibm.xtools.rmps.web.deliver.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.renameChangeset,
					iconClass : "com-ibm-team-vvc-context-icon-rename-changeset",
					onClick: dojo.hitch(this, function() {
						var changeset = this.selected.changeset;
						var name = prompt(this.messages.renamePrompt, changeset.getTitle());
						if (!name || name === changeset.getTitle()) {
							// if user has cancelled the input prompt
							return;
						}
						var waitingDialog = InfoContentUtils.showWaitingDialog();
						var deferred = ChangesetsClient.rename(changeset.getUri(), name);
						deferred.addCallback(this, dojo.hitch(this, function(){
							waitingDialog.close();
							//update the cached context for the popup menu
							var currentContext = ContextUtils.getCurrentContext();
							var children = this._menu.getChildren();
							if (currentContext && children) {
								for (var i = 0; i < children.length; i++) {
									var child = children[i];
									if (child.configuration && (child.configuration.uri === currentContext.getUri())) {
										child.configuration.setTitle(name);
									}
								}							
							}
							
							//update the changeset object name (faster than reloading)
							changeset.setProperty(DCTERMS_TITLE, name);
							var escapedName = jazz.util.html.escape(name);
							this.selected.setLabel(escapedName);
							this._label.innerHTML = escapedName;
							UIUtils.showPageMessage(this.messages.changesetRenamed, 5000);
							dojo.publish(VvcConstants.EVENTS_CHANGESET_RENAMED, [changeset.getUri(), name]);
						}));
						deferred.addErrback(this, dojo.hitch(this, function(error) {
							waitingDialog.close();
							this.displayPopup(dojo.string.substitute(this.messages.failedToRenameChangeset, [changeset.getTitle()]), deferred.ioArgs.xhr.responseText);
						}));
					}),
					isInvisible: function(context) {
						if (context != null && context.isChangeset())
							return true;
						return false;
					},
					
					getDisabledText: dojo.hitch(this, function(context){
						var changeset = ChangesetsClient.getCurrentChangeset();
						if(changeset){
							if (!SecurityUtils.canAdministerOrModifyChangeset(changeset, context)) {
								return SecurityUtils.getNoPermissionChangesetMessage(context.getProperty("controllingAreaName"));
							}
						}
					})
				});				
				this.addChild(this._renameChangesetMenuItem);
			},
			
			_initializeCompleteChangesetMenuItem: function __initializeCompleteChangesetMenuItem() {
				var self = this;
				this._addMenuSectionIfNeeded(this.messages.manageChangesets);
				this._completeChangesetMenuItem = new com.ibm.xtools.rmps.web.deliver.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.completeChangeset,
					iconClass : "com-ibm-team-vvc-context-icon-complete-changeset",
					onClick: dojo.hitch(this, function() {
						var changeset = this.selected.changeset;
						InfoContentUtils.showQuestionDialog(
							this.messages.completeConfirmationTitle,
							dojo.string.substitute(this.messages.completeConfirmationContent, {changesetTitle: changeset.getTitle()}),
							this.messages.completeConfirmationOkButton,
							this.messages.cancelButton,
							dojo.hitch(this, function(){
								var waitingDialog = InfoContentUtils.showWaitingDialog();
								var deferred = ChangesetsClient.complete(changeset.getUri());
								deferred.addCallback(this, dojo.hitch(this, function(){
									waitingDialog.close();
									this.selected.setAttribute("iconClass", "com-ibm-team-vvc-context-completed-changeset");
									dojo.addClass(this._icon, "com-ibm-team-vvc-context-completed-changeset");
									this._toggleMenuItems();
									UIUtils.showPageMessage(this.messages.changesetCompleted, 5000);
									dojo.publish(VvcConstants.EVENTS_CHANGESET_COMPLETED, [changeset.getUri()]);
								}));
								deferred.addErrback(this, dojo.hitch(this, function(error) {
									waitingDialog.close();
									this.displayPopup(dojo.string.substitute(this.messages.failedToCompleteChangeset, [changeset.getTitle()]), deferred.ioArgs.xhr.responseText);
								}));
							})
						);
					}),
					isInvisible: function(context) {
						if (context != null && context.isChangeset() && context.isMutable())
							return true;
						return false;
					},
					
					getDisabledText: dojo.hitch(this, function(context){
						var changeset = ChangesetsClient.getCurrentChangeset();
						if(changeset){
							if (!SecurityUtils.canAdministerOrModifyChangeset(changeset, context)) {
								return SecurityUtils.getNoPermissionChangesetMessage(context.getProperty("controllingAreaName"));
							}
						}
					})
				});
				this.addChild(this._completeChangesetMenuItem);
			},
			
			_initializeCommitChangesetMenuItem: function __initializeCommitChangesetMenuItem() {
				var self = this;
				this._addMenuSectionIfNeeded(this.messages.manageChangesets);
				this._commitChangesetMenuItem = new com.ibm.xtools.rmps.web.deliver.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.commitChangeset,
					iconClass : "com-ibm-team-vvc-context-icon-commit-changeset",
					onClick: dojo.hitch(this, function() {
						var changeset = this.selected.changeset;
						InfoContentUtils.showQuestionDialog(
							this.messages.shareConfirmationTitle,
							dojo.string.substitute(this.messages.shareConfirmationContent, {changesetTitle: changeset.getTitle()}),
							this.messages.shareConfirmationOkButton,
							this.messages.cancelButton,
							dojo.hitch(this, function(){
								var waitingDialog = InfoContentUtils.showWaitingDialog();
								var context = ContextUtils.getCurrentContext();
								if (!context.changesetResource) {
									context.changesetResource = new ChangesetResource();
									context.changesetResource.loadByConfiguration(context);
								}
								var getHead = jazz.client.xhrHead({
									url: context.changesetResource.getUri(),
									headers: {
										referrerUrl: document.location.href
									}
								});
								this.getHead = getHead;
								getHead.then(dojo.hitch(this, function __getHeadOk(response){
									var deliverToStreamUri = this.getHead.ioArgs.xhr.getResponseHeader("deliverToStreamUri");
									if(deliverToStreamUri){
										var redirectRequest = jazz.client.xhrHead({
											url: VvcUtils.getVvcUri() + "/" + decodeURIComponent(deliverToStreamUri)
										});
										
										/*
										 * The JAF proxy does not pass on HTTP 3XX responses properly, instead converting them to 
										 * 200 OK, so the browser does not know to redirect to the new URI.  We must instead 
										 * reimplement this behaviour.  See the JAF defect: 
										 *   331831: JAF Proxy does not relay 3XX responses to the original requester 
										 */
										redirectRequest.then(dojo.hitch(redirectRequest, function __reimplementHttp300Responses(){
											waitingDialog.close();
											var locationHeader = this.ioArgs.xhr.getResponseHeader("location");
											if(locationHeader){
												location.assign(locationHeader + "#/steps/3");
											}
										}), dojo.hitch(this, function __reimplementHttp300ResponsesError(error){
											waitingDialog.close();
											this.displayPopup(dojo.string.substitute(this.messages.failedToCommitChangeset, [changeset.getTitle()]), redirectRequest.ioArgs.xhr.responseText);
										}));										
									} else {
										var deferred = ChangesetsClient.commit(changeset.getUri());
										deferred.addCallback(this, dojo.hitch(this, function(){
											waitingDialog.close();
											UIUtils.showPageMessage(this.messages.changesetCommitted, 5000);
											dojo.publish(VvcConstants.EVENTS_CHANGESET_SHARED, [changeset.getUri()]);
											//expanded context may or may not be null
											this.onContextChange({uri: this.selected.configuration.getParentWorkspaceUri()}, ContextUtils.getExpandedContext());
										}));
										deferred.addErrback(this, dojo.hitch(this, function(error) {
											waitingDialog.close();
											this.displayPopup(dojo.string.substitute(this.messages.failedToCommitChangeset, [changeset.getTitle()]), deferred.ioArgs.xhr.responseText);
										}));
									}
								}));
							})
						);
					}),
					isInvisible: function(context) {
						if (context != null && context.isChangeset())
							return true;
						return false;
					},
					
					getDisabledText: dojo.hitch(this, function(context){
						var changeset = ChangesetsClient.getCurrentChangeset();
						if(changeset){
							if (!SecurityUtils.canAdministerChangeset(changeset, context)) {
								return SecurityUtils.getNoPermissionChangesetMessage(context.getProperty("controllingAreaName"));
							}
						}
					})
				});				
				this.addChild(this._commitChangesetMenuItem);
			},
			
			_initializeDiscardChangesetsMenuItem: function __initializeDiscardChangesetsMenuItem() {
				var self = this;
				this._addMenuSectionIfNeeded(this.messages.manageChangesets);
				this._discardChangesetsMenuItem = new com.ibm.xtools.rmps.web.deliver.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.discardChangeset,
					iconClass : "com-ibm-team-vvc-context-icon-discard-changeset",
					onClick: dojo.hitch(this, function() {
						var changeset = this.selected.changeset;
						InfoContentUtils.showQuestionDialog(
							this.messages.discardConfirmationTitle,
							dojo.string.substitute(this.messages.discardConfirmationContent, {changesetTitle: changeset.getTitle()}),
							this.messages.discardConfirmationOkButton,
							this.messages.cancelButton,
							dojo.hitch(this, function(){
								var waitingDialog = InfoContentUtils.showWaitingDialog();
								var deferred = ChangesetsClient.discard(changeset.getUri());
								deferred.addCallback(this, dojo.hitch(this, function(){
									waitingDialog.close();
									//expanded context may or may not be null
									dojo.publish(VvcConstants.EVENTS_CHANGESET_DISCARDED, [changeset.getUri()]);
									this.onContextChange({uri: this.selected.configuration.getParentWorkspaceUri()}, ContextUtils.getExpandedContext());
								}));
								deferred.addErrback(this, dojo.hitch(this, function(error) {
									waitingDialog.close();
									this.displayPopup(dojo.string.substitute(this.messages.failedToDiscardChangeset, [changeset.getTitle()]), deferred.ioArgs.xhr.responseText);
								}));
							})
						);
					}),
					isInvisible: function(context) {
						if (context != null && context.isChangeset() && context.isMutable())
							return true;
						return false;
					},
					
					getDisabledText: dojo.hitch(this, function(context){
						var changeset = ChangesetsClient.getCurrentChangeset();
						if(changeset){
							if (!SecurityUtils.canAdministerChangeset(changeset, context)) {
								return SecurityUtils.getNoPermissionChangesetMessage(context.getProperty("controllingAreaName"));
							}
						}
					})
				});				
				this.addChild(this._discardChangesetsMenuItem);
				
			},
			
			_initializeLinkWorkItemMenuItem: function __initializeLinkWorkItemMenuItem() {
				var self = this;
				this._addMenuSectionIfNeeded(this.messages.manageChangesets);
				var resultsDeferred = CLMUtils.getFriendRootServices();
				
				var foundCMServiceProvider = false;
				resultsDeferred.then(dojo.hitch(this, function(results) {
					for(var result in results){
						var rootServiceDocument = results[result];
						var cmServiceProviders = rootServiceDocument.getElementsByTagNameNS("http://open-services.net/xmlns/cm/1.0/", "cmServiceProviders");
						if (cmServiceProviders.length > 0) {
							foundCMServiceProvider = true;
							break;
						}	
					}
					if (foundCMServiceProvider && ContextUtils.getCurrentContext().isChangeset()) {
						this._linkWorkItemMenuItem = new com.ibm.xtools.rmps.web.deliver.shared.web.ui.widgets.HideableMenuItem({
							label : this.messages.linkToWorkItemTitle,
							iconClass : "com-ibm-team-vvc-link-to-workitem",
							onClick: function() {
								com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.LinkToWorkItemUtil.linkToWorkItemAction();
							},
							isInvisible: function(context) {
								return (context != null);
							}
							
						});				
						this.addChild(this._linkWorkItemMenuItem);
					}
				}));
			},
			
			_initializeShowChangesetsMenuItem: function __initializeShowChangesetsMenuItem() {
				var self = this;
				this._addMenuSectionIfNeeded(this.messages.manageChangesets);
				var showChangesetsExploreLabelHref = "<a class=\"noDecoration\" href=\"" + this._getShowChangesetsHref() + "\">" + this.messages.showChangesets + "</a>";
				this._showChangesetsMenuItem = new com.ibm.xtools.rmps.web.deliver.shared.web.ui.widgets.HideableMenuItem({
					label : showChangesetsExploreLabelHref,
					iconClass: "com-ibm-team-vvc-changesets-icon",
					onClick: function(e) {
						if((e instanceof MouseEvent && e.which != 2) || (e instanceof KeyboardEvent && (e.which === dojo.keys.ENTER || e.which == dojo.keys.SPACE))){
							self._showChangesets();
						}
					}
				});
				this.addChild(this._showChangesetsMenuItem);
			},
			
			_initializeCompareChangesetsMenuItem: function ___initializeCompareChangesetsMenuItem() {
				var self = this;
				this._addMenuSectionIfNeeded(this.messages.manageChangesets);
				var context = ContextUtils.getCurrentContext();
				if(context.isChangeset()){
					if (!context.changesetResource) {
						context.changesetResource = new ChangesetResource();
						context.changesetResource.loadByConfiguration(context);
					}
					var getHead = jazz.client.xhrHead({
						url: context.changesetResource.getUri(),
					});
					this.getHead = getHead;
					getHead.then(dojo.hitch(this, function __getHeadOk(response){
						var compareWithStreamUri = this.getHead.ioArgs.xhr.getResponseHeader("compareWithStreamUri");
						if(compareWithStreamUri){
							this._compareChangesetToStreamMenuItem = new com.ibm.xtools.rmps.web.deliver.shared.web.ui.widgets.HideableMenuItem({
								label : this.messages.compareToStream,
								iconClass: "com-ibm-team-vvc-compare-with-stream",
								onClick: function() {
									var waitingDialog = InfoContentUtils.showWaitingDialog();
									var redirectRequest = jazz.client.xhrHead({
										url: VvcUtils.getVvcUri() + "/" + decodeURIComponent(compareWithStreamUri)
									});
									
									/*
									 * The JAF proxy does not pass on HTTP 3XX responses properly, instead converting them to 
									 * 200 OK, so the browser does not know to redirect to the new URI.  We must instead 
									 * reimplement this behaviour.  See the JAF defect: 
									 *   331831: JAF Proxy does not relay 3XX responses to the original requester 
									 */
									redirectRequest.then(dojo.hitch(redirectRequest, function __reimplementHttp300Responses(){
										waitingDialog.close();
										var locationHeader = this.ioArgs.xhr.getResponseHeader("location");
										if(locationHeader){
											location.assign(locationHeader + "#/steps/3");
										}
									}), waitingDialog.close)
								}
							});
							this.addChild(this._compareChangesetToStreamMenuItem);
						}
					}));
				}
			},
			
			isCreateChangesetsEnabled: function __isCreateChangesetsEnabled() {
				var deferred = new dojo.Deferred();
				deferred.resolve(true);
				return deferred;
			},
			
			isRenameChangesetEnabled: function __isRenameChangesetEnabled() {
				var deferred = new dojo.Deferred();
				deferred.resolve(true);
				return deferred;
			},
			
			isCompleteChangesetEnabled: function __isCompleteChangesetEnabled() {
				var deferred = new dojo.Deferred();
				deferred.resolve(true);
				return deferred;
			},
			
			isCommitChangesetEnabled: function __isCommitChangesetEnabled() {
				var deferred = new dojo.Deferred();
				deferred.resolve(true);
				return deferred;
			},
			
			isDiscardChangesetEnabled: function __isDiscardChangesetEnabled() {
				var deferred = new dojo.Deferred();
				deferred.resolve(true);
				return deferred;
			},	
			
			isLinkWorkItemEnabled: function __isLinkWorkItemEnabled() {
				var deferred = new dojo.Deferred();
				deferred.resolve(true);
				return deferred;
			},
			
			isShowChangesetsEnabled: function __isShowChangesetsEnabled() {
				var deferred = new dojo.Deferred();
				deferred.resolve(true);
				return deferred;
			},
			
			isCompareChangesetWithStreamEnabled: function __isCompareChangesetWithStreamEnabled(){
				var deferred = new dojo.Deferred();
				deferred.resolve(true);
				return deferred;
			},
			
			_initializeManageChangesets: function __initializeManageChangesets()
			{
				var initManageChangesetsFinished = new dojo.Deferred();
				var dl = new dojo.DeferredList([this.isCreateChangesetsEnabled(), this.isRenameChangesetEnabled(),
				                                this.isCompleteChangesetEnabled(), this.isCommitChangesetEnabled(),
				                                this.isDiscardChangesetEnabled(), this.isLinkWorkItemEnabled(),
				                                this.isShowChangesetsEnabled(), this.isCompareChangesetWithStreamEnabled()]);
				dl.then(dojo.hitch(this, function __initManageChangesets(res) {
					if (res[0][0] && res[0][1]) {
						this._initializeCreateChangesetsMenuItem();
					}
					if (res[1][0] && res[1][1]) {
						this._initializeRenameChangesetMenuItem();
					}
					if (res[2][0] && res[2][1]) {
						this._initializeCompleteChangesetMenuItem();
					}
					if (res[3][0] && res[3][1]) {
						this._initializeCommitChangesetMenuItem();
					}
					if (res[4][0] && res[4][1]) {
						this._initializeDiscardChangesetsMenuItem();
					}
					if (res[5][0] && res[5][1]) {
						this._initializeLinkWorkItemMenuItem();
					}
					if (res[6][0] && res[6][1]) {
						this._initializeShowChangesetsMenuItem();
					}
					if (res[7][0] && res[7][1]) {
						this._initializeCompareChangesetsMenuItem();
					}
					initManageChangesetsFinished.resolve(true);
				}));	
				return initManageChangesetsFinished;
			},
			
			_hasCcmFriends: function __hasCcmFriends() {
				var resultsDeferred = CLMUtils.getFriendRootServices();
				
				var foundCMServiceProvider = false;
				resultsDeferred.addCallback(dojo.hitch(this, function(results){
					for(var result in results){
						var rootServiceDocument = results[result];
						var cmServiceProviders = rootServiceDocument.getElementsByTagNameNS("http://open-services.net/xmlns/cm/1.0/", "cmServiceProviders");
						if (cmServiceProviders.length > 0) {
							foundCMServiceProvider = true;
							break;
						}	
					}
				}));
				return foundCMServiceProvider;
			},
			
			_makePopupContents: function __makePopupContents(configurationPopup, currentContext, expandedContext /*expandedContext could be undefined*/) {
				
				if (!configurationPopup) {
					configurationPopup = this._configurationPopup;
				}
				
				var contextTitle = jazz.util.html.escape(expandedContext ? expandedContext.getTitle() : currentContext.getTitle());
				
				var globalConfiguration = "";
				if (expandedContext) {
					globalConfiguration = "<div>" + this.messages.localConfiguration + currentContext.getTitle() + "</div>";
				}
				
				var configurationSpace = "", parentSnapshot = "", parentWorkspace = "";
				if (!expandedContext) {
					var configurationSpaceName = currentContext.getSpaceName();
					configurationSpaceName = configurationSpaceName ? jazz.util.html.escape(configurationSpaceName) : this.messages.noConfiguration;
					configurationSpace = "<div>" + this.messages.configurationSpace + configurationSpaceName + "</div>";
					
					var parentSnapshotName = currentContext.getParentSnapshotName();
					parentSnapshotName = parentSnapshotName ? jazz.util.html.escape(parentSnapshotName) : this.messages.noConfiguration;
					
					var parentWorkspaceName = currentContext.getParentWorkspaceName();
					parentWorkspaceName = parentWorkspaceName ? jazz.util.html.escape(parentWorkspaceName) : this.messages.noConfiguration;
					
					//don't bother showing the parent snapshot when there is no parent workspace
					if (currentContext.isWorkspace()) {
						var parentSnapshotName = currentContext.getParentSnapshotName();
						parentSnapshotName = parentSnapshotName ? jazz.util.html.escape(parentSnapshotName) : this.messages.noConfiguration;
						parentSnapshot = "<div>" + this.messages.parentSnapshot + parentSnapshotName + "</div>";
					}
					
					parentWorkspace = "<div>" + (currentContext.isWorkspace() ? this.messages.parentWorkspace : this.messages.workspace) + parentWorkspaceName + "</div>";
				}
				
				var descriptionText = expandedContext ? expandedContext.getDescription() : currentContext.getDescription();
				descriptionText = descriptionText ? jazz.util.html.escape(descriptionText) : this.messages.noDescription;
				description = "<div>" + this.messages.description + descriptionText + "</div>";

				var iconClass = expandedContext ? ContextUtils.getExpandedConfigurationIconClass(expandedContext) : ContextUtils.getConfigurationIconClass(currentContext);
				configurationPopup.content.innerHTML =  "<div class = \"configurationPopup-divider\"><div style=\"float:left\"><div class=\"" + iconClass + "\"></div></div><div class = \"configurationPopup-title\">" + contextTitle + "</div>" + "</div>" + globalConfiguration + configurationSpace + parentWorkspace + parentSnapshot + description;
	
				if (currentContext.isChangeset()) {
					
					if (!currentContext.changesetResource) {
						currentContext.changesetResource = new com.ibm.xtools.rmps.web.deliver.shared.core.ui.objects.ChangesetResource();
						currentContext.changesetResource.loadByConfiguration(currentContext);
					}
					
					
					if (currentContext.changesetResource) {
						var callback = new dojo.Deferred();
						CLMUtils.doOSLCQuery(this.providers, currentContext.changesetResource.uri, callback);
						
						callback.then(dojo.hitch(this, function(response){
							var uris = [], anchors = [], nodes = [];
							for(var i = 0; i < response.length; i++){

								var node = dojo.create("div", { "class":"configurationPopup-indent" });
								
								if (i === 0) {
									var mainChangeRequestsNode = dojo.create("div");
									mainChangeRequestsNode.appendChild(dojo.doc.createTextNode(this.messages.associatedWorkItems));
									configurationPopup.content.appendChild(mainChangeRequestsNode);
									
								}
								
								var idString = "-";
								var indexOfSlash = response[i].target.lastIndexOf("/");
								if (indexOfSlash > 0 && indexOfSlash < response[i].target.length - 1) {
									idString = response[i].target.substring(indexOfSlash + 1);
								}
								
								var img = dojo.doc.createElement("img");
								
								var targetUrl = dojo.doc.createTextNode(idString + ": " + response[i].target);
								
								var targetUrlAnchor = dojo.doc.createElement("a");
								targetUrlAnchor.href = response[i].target;
								targetUrlAnchor.appendChild(targetUrl);

								var workItemObj = {
									node: node,
									id: idString
								};
								nodes.push(workItemObj);
								node.appendChild(img);
								node.appendChild(targetUrlAnchor);
								
								uris.push(response[i].target);
								
								var obj = {
									url: response[i].target,
									targetUrl: targetUrl,
									img: img
								};
								anchors.push(obj);
							}
							
							nodes.sort(CLMUtils.idSorter);
							
							for (var i = 0; i < nodes.length; i++) {
								configurationPopup.content.appendChild(nodes[i].node);
							}
							
							var deferredCompact = jazz.client.xhrPost({
								url: net.jazz.ajax.contextRoot() + "/_compactRendering",
								handleAs: "json",
								content: {
									uri: uris
								}
							});

							var context = {
								anchors: anchors,
								messages: this.messages
							};
							
							//_err for individual failures
							deferredCompact.addCallback(context, dojo.hitch(context, function(result){
								for (var i = 0; i < this.anchors.length; i++) {
									var compact = result[this.anchors[i].url];
									var textContent = this.anchors[i].targetUrl.textContent;
									var nodeValue = this.anchors[i].targetUrl.nodeValue; //IE8
									
									if (compact._err) {
										if (textContent) {
											this.anchors[i].targetUrl.textContent = this.messages.unableToLoadTitle;
										} else if (nodeValue) {
											this.anchors[i].targetUrl.nodeValue = this.messages.unableToLoadTitle;
										}
									}
									if (compact && compact.oslcCompact) {
										if (compact.oslcCompact.title) {
											if (textContent) {
												this.anchors[i].targetUrl.textContent = compact.oslcCompact.title;
											} else if (nodeValue) {
												this.anchors[i].targetUrl.nodeValue = compact.oslcCompact.title;
											}
										} else {
											if (textContent) {
												this.anchors[i].targetUrl.textContent = this.messages.unableToLoadTitle;
											} else if (nodeValue) {
												this.anchors[i].targetUrl.nodeValue = this.messages.unableToLoadTitle;
											}
										}
										
										if (compact.oslcCompact.icon) {
											this.anchors[i].img.src = compact.oslcCompact.icon;
										}
									} else {
										if (textContent) {
											this.anchors[i].targetUrl.textContent = this.messages.unableToLoadTitle;
										} else if (nodeValue) {
											this.anchors[i].targetUrl.nodeValue = this.messages.unableToLoadTitle;
										}
									}
								}
							}));
							
							//all failed
							deferredCompact.addErrback(context, dojo.hitch(context, function(result){
								for (var i = 0; i < this.anchors.length; i++) {
									if (this.anchors[i].targetUrl.textContent) {
										this.anchors[i].targetUrl.textContent = this.messages.unableToLoadTitle;
									} else if (this.anchors[i].targetUrl.nodeValue) {
										this.anchors[i].targetUrl.nodeValue = this.messages.unableToLoadTitle;
									}
								}
							}));						

						}));
						
					}

				}
			},
			
			openCurrentConfigurationMenuItemEnabled: function __openCurrentConfigurationMenuItemEnabled() {
				var deferred = new dojo.Deferred();
				deferred.resolve(true);
				return deferred;
			},
			
			exploreMenuItemEnabled: function __exploreMenuItemEnabled() {
				var deferred = new dojo.Deferred();
				deferred.resolve(true);
				return deferred;
			},
			
			_initializeOpenCurrentConfigurationMenuItem: function __initializeOpenCurrentConfigurationMenuItem() {
				var self = this;
				this._addMenuSectionIfNeeded(this.messages.manageConfigurations);
				var title = dojo.string.substitute(this.messages.openCurrentConfiguration, {
					configTitle: ContextUtils.getCurrentContext().getTitle() // Should never be null!
				});
				var openCurrentConflabelHref = "<a class=\"noDecoration\" href=\"" + ContextUtils.getCurrentContext().uri + "\">" + title + "</a>"; 
				this._openCurrentConfigurationMenuItem = new com.ibm.xtools.rmps.web.deliver.shared.web.ui.widgets.HideableMenuItem({
					label : openCurrentConflabelHref,
					iconClass : "com-ibm-team-vvc-context-icon-edit",
					onClick: function __onClickEditCurrentConf(e) {
						if((e instanceof MouseEvent && e.which != 2) || (e instanceof KeyboardEvent && (e.which === dojo.keys.ENTER || e.which == dojo.keys.SPACE))){
							self._openCurrentConfiguration();
						}
					},
					isInvisible: function(context) {
						return !context.isChangeset();
					}
				});
				this.addChild(this._openCurrentConfigurationMenuItem);
			},
			
			_initializeExploreMenuItem: function __initializeExploreMenuItem()
			{
				var self = this;
				this._addMenuSectionIfNeeded(this.messages.manageConfigurations);
				var explorerLabelHref = "<a class=\"noDecoration\" href=\"" + this._getExplorerHref() + "\">" + this.messages.exploreConfigurations + "</a>";
				var exploreMenuItem = new com.ibm.xtools.rmps.web.deliver.shared.web.ui.widgets.HideableMenuItem({
					label : explorerLabelHref,
					iconClass: "com-ibm-team-vvc-explorer-icon",
					onClick: function(e) {
						if((e instanceof MouseEvent && e.which != 2) || (e instanceof KeyboardEvent && (e.which === dojo.keys.ENTER || e.which == dojo.keys.SPACE))){
							self._showExplorer();
						}
					}
				});	
				this.addChild(exploreMenuItem);
			},
			
			_initializeManageConfigurations: function __initializeManageConfigurations()
			{
				var initManageConfigurationsFinished = new dojo.Deferred();
				var dl = new dojo.DeferredList([this.openCurrentConfigurationMenuItemEnabled(),this.exploreMenuItemEnabled()]);
				dl.then(dojo.hitch(this, function __initManageConfigurations(res) {
					if (res[0][0] && res[0][1]) {	
						this._initializeOpenCurrentConfigurationMenuItem();
					}
					if (res[1][0] && res[1][1]) {
						this._initializeExploreMenuItem();
					}
				}));
				initManageConfigurationsFinished.resolve(true);
				return initManageConfigurationsFinished;
			},
			
			globalConfigMoreMenuItemEnabled: function __globalConfigMoreMenuItemEnabled() {
				var deferred = new dojo.Deferred();
				deferred.resolve(true);
				return deferred;
			},
			
			globalConfigAddToGlobalConfigurationMenuItemEnabled: function __globalConfigAddToGlobalConfigurationMenuItemEnabled() {
				var deferred = new dojo.Deferred();
				deferred.resolve(true);
				return deferred;
			},
			
			_initGlobalConfigMoreMenu: function __initGlobalConfigMoreMenu() {
				this._addMenuSectionIfNeeded(this.messages.globalConfigurations);
				var self = this;
				var moreMenuItem = new com.ibm.xtools.rmps.web.deliver.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.expandContext,
					iconClass : "com-ibm-team-vvc-context-global-configuration-select",
					onClick: function() {
						var args = {};
						var lastContext = ContextUtils.getCurrentContext();	
						args.findParentsOfUri = lastContext.uri;
						args.dialogTitle = self.messages.expandContextDialogTitle;
						args.pureGlobalConfigurationsOnly = true;
						args.iconPath = dojo.moduleUrl("com.ibm.xtools.rmps.web.deliver.shared.web", "graphics/globalconfiguration_select.gif");
						var picker = new com.ibm.xtools.rmps.web.deliver.shared.web.ui.picker.WorkspacePickerDialog(args);
						picker.onOk = function(configuration) {
							if (!self._isValid(configuration))							
								return;
							ContextUtils.setCurrentContext(null, configuration).then(function __setCurrentContextComplete() {
								self._initializeSelectedMenuItem();
								
								if (self.onContextChange) {
									self.onContextChange(null, configuration);
								}
							},
							function __setCurrentContextFailed(err) {
								self.displayPopup(self.messages.failedToObtainGcContributions);
							});
						};
					},
					isInvisible: function(context) {
						return true;
					}
				});				
				this.addChild(moreMenuItem);
			},
			
			_initGlobalConfigAddToGCMenu: function __initGlobalConfigAddToGCMenu() {
				this._addMenuSectionIfNeeded(this.messages.globalConfigurations);
				var self = this;
				this._addToGlobalConfigurationMenuItem = new com.ibm.xtools.rmps.web.deliver.shared.web.ui.widgets.HideableMenuItem({
					label : this.messages.addToGlobalConfiguration,
					iconClass : "com-ibm-team-vvc-context-global-configuration-add",
					onClick: function() {
						if (self._checkValid()) {
							self._addToGlobalConfiguration();
						}
					},
					isInvisible: dojo.hitch(this, function(context) {
						return true; //this method should really be isVisible
					})

					//TODO permissions
//					getDisabledText: dojo.hitch(this, function(context){
//						if (!context.permissions || dojo.indexOf(context.permissions, SecurityConstants.permissions.createOrModifyWorkspace) < 0) {
//							return SecurityUtils.getNoPermissionMessage(context.getProperty("controllingAreaName"));
//						}
//						return null;
//					})
				});		
				this.addChild(this._addToGlobalConfigurationMenuItem);		
			},
			
			_initializeGlobalConfigurations: function __initializeGlobalConfigurations() {
				var initFinished = new dojo.Deferred();
				if(!areGcsEnabled){
					// GC's disabled - don't add this step.
					return;
				}
				var dl = new dojo.DeferredList([this.globalConfigMoreMenuItemEnabled(), this.globalConfigAddToGlobalConfigurationMenuItemEnabled()]);
				dl.then(dojo.hitch(this, function __initGlobalConfigurations(res) {
					if (res[0][0] && res[0][1]) {	
						this._initGlobalConfigMoreMenu();
					}
					if (res[1][0] && res[1][1]) {
						this._initGlobalConfigAddToGCMenu();
					}
					initFinished.resolve(true);
				}));
				return initFinished;
			},
			
			_toggleMenuItems: function __toggleMenuItems()
			{
				var currentContext = ContextUtils.getCurrentContext();
				if(currentContext){
					var menuItems = this._menu.getChildren();
					dojo.forEach(menuItems, dojo.hitch(this, function(item, index){
						if(item.isInvisible){
							var visible = item.isInvisible(currentContext);
							item.setVisible(!visible);
							if(visible && item.getDisabledText){									
								dojo.when(item.getDisabledText(currentContext), dojo.hitch(this, function(result){
									if (result) {
										item.set('disabled', true);
										item.set('title', result);
									}
								}));
							}
						}
					}));
				}
			},
			
			_constructMenuItem: function __constructMenuItem(configuration, expandedContext) {
				var menuItem = new com.ibm.xtools.rmps.web.deliver.shared.web.ui.widgets.HideableMenuItem({
					label : UIUtils.truncateString(jazz.util.html.escape((expandedContext && !configuration.isChangeset()) ? expandedContext.getTitle() : configuration.getTitle())),
					"class" : "com-ibm-team-vvc-context-menuItem",
					iconClass : (expandedContext && !configuration.isChangeset()) ? ContextUtils.getExpandedConfigurationIconClass(expandedContext) : ContextUtils.getConfigurationIconClass(configuration),
					configuration: configuration,
					expandedContext: expandedContext,
					changeset: ChangesetsClient.getCurrentChangeset(),
					isConfiguration: true
				});		
				return menuItem;
			},
			
			_addMenuSectionIfNeeded: function __addMenuSectionIfNeeded(title) {
				if (!menuSectionMap[title]) {
					this._menu.addChild(new dijit.MenuSeparator());
					this._menu.addChild(new jazz.ui.menu.NodeMenuItem({isTitle: true, label: title}));
					menuSectionMap[title] = true;
				}
			},
			
			onChange: function(item)
			{
				if (item.configuration != null)
					ContextUtils.setCurrentContext(item.configuration, item.expandedContext).then(dojo.hitch(this, function __currentContextSetInOnChange() {
						this._toggleMenuItems();
						
						if (this.onContextChange) {
							this.onContextChange(item.configuration, item.expandedContext);
						}
					}),
					dojo.hitch(this, function __currentContextFailedInOnChange(err) {
						this.displayPopup(this.messages.failedToObtainGcContributions);
					}));				
			},

			_onClick: function(event) {
				this._menuPopup.open();
			},

			_onMouseOver: function(event) {
				this._over = true;
				this._toggleActiveClass(true);
			},
			
			_onMouseOut: function(event) {
				this._over = false;
				this._toggleActiveClass(false);
			},
			
			_menuProvider: function()
			{
				return this._menu;
			},
			
			_toggleActiveClass: function __toggleActiveClass(active) {
				dojo.toggleClass(this.domNode, 'active', active || this._open || this._over);
			},
			
			_openCurrentConfiguration: function __openCurrentConfiguration(uri)  {
				if(uri){
					location.assign(uri);
				} else {
					location.assign(ContextUtils.getCurrentContext().uri);
				}
			},
						
			_newSnapshot:function()
			{
				this._newConfigurationForm({type: "snapshot"});
			},
			
			_newWorkspace: function __newWorkspace()
			{
				this._newConfigurationForm({type: "workspace"});
			},
			
			_newConfigurationForm: function __newConfigurationForm(args){
				if (VvcUtils.getVvcUri())
				{
					var form = new com.ibm.xtools.rmps.web.deliver.shared.web.ui.configurations.CreateConfigurationForm(args);
					
					form.loadConfiguration = dojo.hitch(this, function(configuration){	
						ContextUtils.setCurrentContext(configuration).then(dojo.hitch(this, function __currentContextSetInNewConfigurationForm() {
							this._initializeSelectedMenuItem();
							
							if (this.onContextChange) {
								this.onContextChange(configuration);
							}
						}),
						dojo.hitch(this, function __currentContextFailedInNewConfigurationForm(err) {
							this.displayPopup(this.messages.failedToObtainGcContributions);
						}));
					});
				}
			},
			
			_addToGlobalConfiguration: function __addToGlobalConfiguration()
			{
				var args= {type: ContextUtils.getCurrentContext().isWorkspace ? "workspace" : "snapshot"};
				var form = new com.ibm.xtools.rmps.web.deliver.shared.web.ui.configurations.AddToGlobalConfigurationForm(args);
				
				form.loadConfiguration = dojo.hitch(this, function(configuration){	
					ContextUtils.setCurrentContext(configuration).then(dojo.hitch(this, function __currentContextSetInAddToGc() {
						this._initializeSelectedMenuItem();
						
						if (this.onContextChange) {
							this.onContextChange(configuration);
						}
					}),
					dojo.hitch(this, function __currentContextFailedInAddToGC(err) {
						this.displayPopup(this.messages.failedToObtainGcContributions);
					}));
				});
			},
			
			_showExplorer: function __showExplorer()
			{
				if (VvcUtils.getVvcUri() != null)
				{
					window.location.href = this._getExplorerHref();
				}
			},
			
			// Do not propagate this!
			_explorerHref: null,
			_getExplorerHref: function __getExplorerHref(){
				if(!this._explorerHref){
					this._explorerHref = VvcUtils.getVvcUri() + "/web"; //?" + this.getUriOptions() + "#action=" + VvcConstants.EXPLORE_CONFIGURATIONS;		
				}
				return this._explorerHref;
			},
			
			_showChangesets: function __showChangesets()
			{
				if (VvcUtils.getVvcUri() != null)
				{
					window.location.href = this._getShowChangesetsHref();
				}
			},
			
			_getShowChangesetsHref : function __getShowChangesetsHref() {
				var context = ContextUtils.getCurrentContext();
				var actionParams = {
					action: VvcConstants.EDIT_WORKSPACE,
					tab: VvcConstants.CHANGESETS_TAB,
					configurationUri: context.isChangeset() ? context.getBase() : context.uri
				};
				var queryParams = {
					debug: dojo.config.isDebug ? true : null
				};
				return VvcUtils.getVvcUri() + "/web?" + dojo.objectToQuery(queryParams) + "#" + dojo.objectToQuery(actionParams);
			},
			
			// sub class can override to include more additional parameters
			getUriOptions: function __getUriOptions(returnOnlyWorkspace) 
			{
				var uriOptions = {};
				var currentContext = ContextUtils.getCurrentContext();
				var contextUri;
				uriOptions.spaceUri = ContextUtils.getSpace();
				
				if (currentContext != null){
					if (returnOnlyWorkspace && !currentContext.isMutable()){
						// want only workspace and snapshots are not mutable
						contextUri = currentContext.getParentWorkspaceUri();
					} else if (returnOnlyWorkspace && currentContext.isChangeset()){
						// want only workspace and changesets have a Change Set dimension 
						// and their owning workspace is their base
						contextUri = currentContext.getBase();
					} else {
						contextUri = currentContext.uri;
					}
					uriOptions.contextUri = contextUri;
				}
				return dojo.objectToQuery(uriOptions);
			},
			
			displayPopup: function(message, rdfResponse) {				
				var details = this.parseRdfResponse(rdfResponse);
				var args = {
					type: details[0] ? details[0].code : "",
					summary: message,
					details: details,
				};
				InfoContentUtils.showDialog(com.ibm.xtools.rmps.web.deliver.shared.web.ui.context.ConfigurationContextBannerButtonMessageDialog, this.messages.operationFailed, args);
			},
			
			parseRdfResponse: function __parseRdfResponse(rdfResponse) {
				if(!rdfResponse || rdfResponse === ""){
					return [];
				}
				var responseDoc = dojox.xml.DomParser.parse(rdfResponse);
				var errorMessage = responseDoc.byName("err:errorMessage")[0].childNodes[0].nodeValue;
				var errorSeverity = responseDoc.byName("err:errorSeverity")[0].childNodes[0].nodeValue;
				var details = {
					message: errorMessage,
					code: jazz.ui.MessageBox.STATUS_TYPES[errorSeverity],
				};
				return [details];
			},
		}
	);
	
	dojo.declare(
			'com.ibm.xtools.rmps.web.deliver.shared.web.ui.context.ConfigurationContextBannerButtonMessageDialog', //$NON-NLS-1$
			[com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.InfoContent], {
			_dialog: null,
			_arguments: null,
			constructor: function(msgBoxArgs) {
				this._arguments = msgBoxArgs;
				this.contentNode = ""; //set it blank intentionally
			},
	
			postCreate: function() {
				this.inherited(arguments);
				var messageBox = new jazz.ui.MessageBox();
				messageBox.showMessage(this._arguments);
				dojo.style(messageBox.domNode.firstElementChild, "backgroundColor", "transparent");
				dojo.style(messageBox.domNode.firstElementChild, "border", "inherit");
				dojo.style(messageBox.domNode.firstElementChild, "color", "inherit");
				this._dialogContent.removeChild(this._dialogContent.lastElementChild);
				dojo.place(messageBox.domNode, this._dialogContent, 'first');				
			},	
		}
	);

})();