/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2015. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/

(function(){
dojo.provide("rmps.app.ui.internal.changesets.ChangeSetLinksTable");
dojo.require("com.ibm.team.repository.web.configurationManagement.grid.CmTreeGrid");
dojo.require("com.ibm.team.repository.web.configurationManagement.grid.CmTreeGridTable");
dojo.require("rmps.core.RootServices");

dojo.require("com.ibm.team.repository.web.configurationManagement.notifications.CmNotificationFactory");
var CmNotificationFactory = com.ibm.team.repository.web.configurationManagement.notifications.CmNotificationFactory;

dojo.require("rmps.app.ui.internal.util.ChangesetResourceUiNode");
var ChangesetResourceUiNode = rmps.app.ui.internal.util.ChangesetResourceUiNode;

dojo.require("rmps.app.ui.internal.editors.views.link.LinkClient");
var LinkClient = rmps.app.ui.internal.editors.views.link.LinkClient;

dojo.require("jazz.ui.ResourceLink");
var ResourceLink = jazz.ui.ResourceLink;

dojo.require("rmps.app.ui.internal.util.CLMUtil");
var CLMUtil = rmps.app.ui.internal.util.CLMUtil;

dojo.require("rmps.ui.util.UIUtil");
var UIUtil = rmps.ui.util.UIUtil;

dojo.requireLocalization("rmps.app.ui.internal.pages", "DmChangeSetEditorMessages");
var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages", "DmChangeSetEditorMessages");

var CM_TRACKS_CHANGE_SET = "http://open-services.net/ns/cm#tracksChangeSet";
											  
dojo.declare("rmps.app.ui.internal.changesets.ChangeSetLinksTable", [com.ibm.team.repository.web.configurationManagement.grid.CmTreeGrid], {
	
	_changesetUri: null, 
	_changesetCache: null,
	
	constructor: function (args){
		this._changesetUri = args.changesetUri;
	},
	
	destroy: function __destroy(){
		this._gc.destroys(this._changesetCache);
		this.inherited(arguments);
		this._changesetUri = null;
		this._changesetCache = null;
	},
	
	buildTreeGridTableNode: function __buildTreeGridTableNode(){
		this._gridTable = new rmps.app.ui.internal.changesets.ChangeSetLinksTable._grid();
		return this._gridTable.domNode;
	},
	
	refresh: function __refresh(){
		this._gridTable.refresh();
		
		var getLinksDeferred = LinkClient.getIncomingLinksTo(this._changesetUri, CM_TRACKS_CHANGE_SET, true);
		UIUtil.showLoadingMessage();
		getLinksDeferred.then(dojo.hitch(this, function(links){
			if(links && links.length > 0) {
				for(var i = 0; i < links.length; i++) {
					var link = links[i];
					var data = {
						displayNameHref : link.subject,
						displayName : link.title,
						changesetUri : this._changesetUri,
						subject : link.subject
					};
					this._gridTable.addRow(data);
				}
			}
			UIUtil.hideLoadingMessage();
		}), dojo.hitch(this, function(error){
			UIUtil.hideLoadingMessage();
			CmNotificationFactory.createXhrError({xhrError: error, ioArgs: error.response.options.ioArgs}).dispatch();
		}));
	}
	
});

dojo.declare("rmps.app.ui.internal.changesets.ChangeSetLinksTable._grid", [com.ibm.team.repository.web.configurationManagement.grid.CmTreeGridTable], {
		
	registerRowBuilders: function __registerRowBuilders(){
		builders = [];
		builders.push(this._linkActionRowBuilder());
		builders.push(this._linkTitleRowBuilder());
		return builders;
	},
	
	_linkTitleRowBuilder: function _linkTitleRowBuilder(tr, data, table){
		var builder = {
			columnName: messages.links,
			columnClass: "linksColumn",
			builderFunc: dojo.hitch(this, function __builderFunc(tr, data, table){
				var readyDeferred = new dojo.Deferred();
				var superReadyDeferred = this._defaultBuilder(tr, data, table);
				dojo.when(superReadyDeferred, dojo.hitch(this, function __superReadyDeferredWhen(td){					
					var gc = tr._gc;		
					var resourceNode = new ChangesetResourceUiNode({
						"data": data
					});
					td.appendChild(resourceNode.domNode);
					
					readyDeferred.resolve(td);
				}));
				return readyDeferred;
			})
		};
		return builder;
	},
	
	_linkActionRowBuilder: function _linkActionRowBuilder(tr, data, table){
		var builder = {
			columnName: messages.actions,
			columnClass: "actionsColumn",
			builderFunc: dojo.hitch(this, function __builderFunc(tr, data, table){
				var readyDeferred = new dojo.Deferred();
				var superReadyDeferred = this._defaultBuilder(tr, data, table);
				dojo.when(superReadyDeferred, dojo.hitch(this, function __superReadyDeferredWhen(td){					
					var btn = new jazz.ui.toolbar.Button({
                        imgClass: "icons-changeset-discard",
                        title: messages.removeLink, 
                        onClick: dojo.hitch(this, function() {
                            var link = {
                                    subject: data.subject,
                                    object:  data.changesetUri,
                                    linkType: CM_TRACKS_CHANGE_SET
                            };
                            LinkClient.deleteDerivedLink(link)
                                              .then(dojo.hitch(tr, tr._removeRow));
                        })
                    });
					td.appendChild(btn.domNode);
					readyDeferred.resolve(td);
				}));
				return readyDeferred;
			})
		};
		return builder;
	}
});

}());