/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2015. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/

(function(){
dojo.provide("rmps.app.ui.internal.changesets.ChangeSetResourcesTable");
dojo.require("com.ibm.team.repository.web.configurationManagement.grid.CmTreeGrid");
dojo.require("com.ibm.team.repository.web.configurationManagement.grid.CmTreeGridTable");
dojo.require("rmps.core.RootServices");

dojo.require("com.ibm.team.repository.web.configurationManagement.notifications.CmNotificationFactory");
var CmNotificationFactory = com.ibm.team.repository.web.configurationManagement.notifications.CmNotificationFactory;

dojo.require("rmps.app.ui.internal.util.ChangesetResourceUiNode");
var ChangesetResourceUiNode = rmps.app.ui.internal.util.ChangesetResourceUiNode;

dojo.require("jazz.ui.ResourceLink");
var ResourceLink = jazz.ui.ResourceLink;

dojo.require("rmps.app.ui.internal.util.CLMUtil");
var CLMUtil = rmps.app.ui.internal.util.CLMUtil;

dojo.require("rmps.ui.util.UIUtil");
var UIUtil = rmps.ui.util.UIUtil;

dojo.requireLocalization("rmps.app.ui.internal.pages", "DmChangeSetEditorMessages");
var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages", "DmChangeSetEditorMessages");

dojo.declare("rmps.app.ui.internal.changesets.ChangeSetResourcesTable", [com.ibm.team.repository.web.configurationManagement.grid.CmTreeGrid], {
	
	_changesetUri: null, 
	_changesetCache: null,
	
	constructor: function (args){
		this._changesetUri = args.changesetUri;
	},
	
	destroy: function __destroy(){
		this._gc.destroys(this._changesetCache);
		this.inherited(arguments);
		this._changesetUri = null;
		this._changesetCache = null;
	},
	
	buildTreeGridTableNode: function __buildTreeGridTableNode(){
		this._gridTable = new rmps.app.ui.internal.changesets.ChangeSetResourcesTable._grid();
		return this._gridTable.domNode;
	},
	
	refresh: function __refresh(){
		this._gridTable.refresh();
		
		var chagesetId = CLMUtil.getIdFromUri(this._changesetUri); 
		var urlParams = {
			resources: true
		};
		
		var getResourcesDeferred = jazz.client.xhrGet({
			url: rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "dm-changesets") + "/" + chagesetId + "?" + dojo.objectToQuery(urlParams)
		});
		UIUtil.showLoadingMessage();
		getResourcesDeferred.then(dojo.hitch(this, function(results){
			if(results){
				var jsonResults = dojo.fromJson(results);
				if(jsonResults.length > 0){
					this._gridTable.addRows(jsonResults);
					UIUtil.hideLoadingMessage();
					return;
				} else {
					UIUtil.hideLoadingMessage();
				}
			}			
		}), dojo.hitch(this, function(error){
			UIUtil.hideLoadingMessage();
			CmNotificationFactory.createXhrError({xhrError: error, ioArgs: error.response.options.ioArgs}).dispatch();
		}));
	}
	
});


dojo.declare("rmps.app.ui.internal.changesets.ChangeSetResourcesTable._grid", [com.ibm.team.repository.web.configurationManagement.grid.CmTreeGridTable], {
		
	registerRowBuilders: function __registerRowBuilders(){
		builders = [];
		builders.push(this._resourceTitleRowBuilder());
		builders.push(this._resourceStatusRowBuilder());
		builders.push(this._resourceVersionRowBuilder());
		return builders;
	},
	
	_resourceTitleRowBuilder: function __resourceTitleRowBuilder(tr, data, table){
		var builder = {
			columnName: messages.resource,
			columnClass: "conceptColumn",
			builderFunc: dojo.hitch(this, function __builderFunc(tr, data, table){
				var readyDeferred = new dojo.Deferred();
				var superReadyDeferred = this._defaultBuilder(tr, data, table);
				dojo.when(superReadyDeferred, dojo.hitch(this, function __superReadyDeferredWhen(td){					
					var gc = tr._gc;		
					var resourceNode = new ChangesetResourceUiNode({
						"data": data
					});
					td.appendChild(resourceNode.domNode);
					
					readyDeferred.resolve(td);
				}));
				return readyDeferred;
			})
		};
		return builder;
	},
	
	_resourceVersionRowBuilder: function __resourceVersionRowBuilder(tr, data, table){
		var builder = {
			columnName: messages.version,
			columnClass: "versionColumn",
			builderFunc: dojo.hitch(this, function __builderFunc(tr, data, table){
				var readyDeferred = new dojo.Deferred();
				var superReadyDeferred = this._defaultBuilder(tr, data, table);
				dojo.when(superReadyDeferred, dojo.hitch(this, function __superReadyDeferredWhen(td){					
					if(data.version){	
						var gc = tr._gc;					
						var title = data.version;
						var lastIndexOfSlash = title.lastIndexOf("--");
						if(lastIndexOfSlash >= 0){
							title = title.substring(lastIndexOfSlash + 2);
						}
						
						var titleNode = dojo.create("span", {
							innerHTML: title,
							title: data.version
						}, td);
						gc.destroys(titleNode);
					}
					readyDeferred.resolve(td);
				}));
				return readyDeferred;
			})
		};
		return builder;
	},
	
	_resourceStatusRowBuilder: function(tr, data, table){
		var builder = {
			columnName: messages.status,
			columnClass: "statusColumn",
			builderFunc: dojo.hitch(this, function __builderFunc(tr, data, table){
				var readyDeferred = new dojo.Deferred();
				var superReadyDeferred = this._defaultBuilder(tr, data, table);
				dojo.when(superReadyDeferred, dojo.hitch(this, function __superReadyDeferredWhen(td){					
					if(data.status){
						var gc = tr._gc;					
						var titleNode = dojo.create("div", {
							innerHTML: data.status,
							title: data.status
						}, td);
						gc.destroys(titleNode);
					}
					readyDeferred.resolve(td);
				}));
				return readyDeferred;
			})
		};
		return builder;
	}
});

}());
