/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/

(function DmChangesetLinkWidget(){
dojo.provide("rmps.app.ui.internal.changesets.DmChangesetLinkWidget");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");

dojo.require("com.ibm.team.repository.web.configurationManagement.changesets.CmChangesetLinkWidget");

dojo.require("com.ibm.team.repository.web.configurationManagement.CmTools");
var CmTools = com.ibm.team.repository.web.configurationManagement.CmTools;

dojo.require("jazz.ui.ResourceLink");
var ResourceLink = jazz.ui.ResourceLink;

dojo.require("com.ibm.team.repository.web.configurationManagement._utils._deprecated.CLMUtils");
var CLMUtils = com.ibm.team.repository.web.configurationManagement._utils._deprecated.CLMUtils;

dojo.requireLocalization("com.ibm.team.repository.web.configurationManagement.changesets", "ChangesetsPageMessages");
var Messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.configurationManagement.changesets", "ChangesetsPageMessages");

dojo.declare("rmps.app.ui.internal.changesets.DmChangesetLinkWidget", [com.ibm.team.repository.web.configurationManagement.changesets.CmChangesetLinkWidget], {
	
	_changeset: null,
	
	constructor: function(changeset){
		this._changeset = changeset;
	},
	
	postCreate: function(){
		this._showAuthenticationNode();
		var authenticationDeferred = new dojo.Deferred();
		CLMUtils.authenticate(authenticationDeferred);
		authenticationDeferred.then(dojo.hitch(this, function(result){
			this._showLoadingNode();
			this._getWorkItemLinks().then(dojo.hitch(this, function(results){
				this._processWorkitemResults(results);
			}), dojo.hitch(this, function(error){
				this._showErrorNode();
			}));
		}), dojo.hitch(this, function(error){
			this._showErrorNode();
		}));
	},
	
	_showAuthenticationNode: function(){
		dojo.empty(this.domNode);
		dojo.create("span", {
			innerHTML: Messages.waitingForAuthentication,
			"class": "noResultsNode"
		}, this.domNode);
	},
	
	_showLoadingNode: function(){
		dojo.empty(this.domNode);
		dojo.create("span", {
			innerHTML: Messages.loading,
			"class": "noResultsNode"
		}, this.domNode);
	},
	
	_showNoResultsFoundNode: function(){
		dojo.empty(this.domNode);
		dojo.create("span", {
			innerHTML: Messages.noWorkitemLinks,
			"class": "noResultsNode"
		}, this.domNode);
	},
	
	_showErrorNode: function(){
		dojo.empty(this.domNode);
		dojo.create("span", {
			innerHTML: Messages.error,
			"class": "error"
		}, this.domNode);
	},
	
	_getWorkItemLinks: function(){
		var urlArgs = {
			changeset: this._changeset.uri,
			consumerUrl: CmTools.getVvcUri()
		};
		
		var getLinksDeferred = jazz.client.xhrGet({
			headers: {
				"Content-Type": "application/x-oslc-common-link+xml",
				"Accept": "application/atom+json"
			},
			url: CmTools.getVvcUri() + "/changesetLinks?" + dojo.objectToQuery(urlArgs),
			handleAs: "json",
			hasBody: false
		});
		return getLinksDeferred;
	},
	
	_sortWorkItemLinks: function(links){
		var sorted = [];
		var toBeAdded;
		while(links.length > 0){
			toBeAdded = links.pop();
			var targetAdding = toBeAdded.target;
			var numberAdding = targetAdding.substring(dojo.lastIndexOf(targetAdding, '/') + 1);
			for(var i = 0; i < sorted; i++){
				var targetAdded = sorted[i];
				var numberAdded = targetAdded.substring(dojo.lastIndexOf(targetAdded, '/') + 1);
				if(numberAdding < numberAdded){
					sorted.splice(i, 1, toBeAdded);
					toBeAdded = null;
					break;
				}
			}
			
			if(toBeAdded){
				sorted.push(toBeAdded);
			}
		}
		return sorted;
	},
	
	_processWorkitemResults: function __processWorkitemResults(results){
		if(results && results.length > 0){
			dojo.empty(this.domNode);
			var sorted = this._sortWorkItemLinks(results);
			for(var i = 0; i < sorted.length; i++){
				var widget = new com.ibm.team.repository.web.configurationManagement.changesets.CmChangesetLinkWidget._workItemWidget(sorted[i]);
				this.domNode.appendChild(widget.domNode);
				if(i < (sorted.length - 1)){
					dojo.create("span", {
						innerHTML: ", "
					}, this.domNode);
				}
			}
		} else {
			this._showNoResultsFoundNode();
		}
	}
	
});

dojo.declare("com.ibm.team.repository.web.configurationManagement.changesets.CmChangesetLinkWidget._workItemWidget", dijit._Widget, {

	_workItem: null,
	_workItemDetails: null,
	_showAbbreviation: null,
	
	constructor: function(workItem){
		this._workItem = workItem;
		this._showAbbreviation = true;
	},
	
	postCreate: function(){
		this.domNode = dojo.create("span");
		this._showCompactNode();
	},
	
	_showCompactNode: function(){
		this._showAbbreviation = true;
		this._refresh();
	},
	
	_showFullNode: function(){
		this._showAbbreviation = false;
		this._refresh();
	},
	
	_refresh: function(){
		dojo.empty(this.domNode);
		var target = this._workItem.target;
		var linkIconNode = dojo.create("span", {
			innerHTML: "&nbsp;"
		}, this.domNode);
		var title = target.substring(dojo.lastIndexOf(target, '/') + 1);
		var link = new ResourceLink({
			uri: this._workItem.target,
			label: title,
			iconNode: linkIconNode,
			useAbbreviation: this._showAbbreviation
		});
		this.domNode.appendChild(link.domNode);
	},
	
	_getWorkitemDetails: function(){
		if(this._workItemDetails){
			return this._getWorkitemDetails();
		}
		var workItemDef = jazz.client.xhrGet({
			url: links[i].target + "?oslc.properties=dcterms:title",
			headers: {
				"Accept": "application/rdf+xml",
				"OSLC-Core-Version": "2.0"
			},
			handleAs: "xml", //$NON-NLS-1$
		});
		workItemDef.then(dojo.hitch(this, function(details){
			this._workItemDetails = details;
		}), dojo.hitch(this, function(error){
			CmNotificationFactory.createXhrError({xhrError: response, ioArgs: ioArgs}).dispatch();
		}));
		return workItemDef;
	}

});

})();