/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2014. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/

(function WorkitemLinksTable(){
	dojo.provide("com.ibm.team.vvc.server.core.ui.internal.editors.WorkitemLinksTable");
	dojo.require("com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGrid");
	dojo.require("com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGridTable");

	dojo.require("jazz.ui.ResourceLink");
	var ResourceLink = jazz.ui.ResourceLink;

	dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
	var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

	dojo.require("com.ibm.team.vvc.shared.oslc.web.utils.CLMUtils");
	var CLMUtils = com.ibm.team.vvc.shared.oslc.web.utils.CLMUtils;

	dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.VvcButtonFactory");
	var VvcButtonFactory = com.ibm.team.vvc.server.core.ui.internal.util.VvcButtonFactory;

	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.editors", "VvcEditorMessages");
	var messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.editors", "VvcEditorMessages");

	dojo.declare("com.ibm.team.vvc.server.core.ui.internal.editors.WorkitemLinksTable", [com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGrid], {

		_changesetUri: null,

		constructor: function (args){
			this._changesetUri = args.changesetUri;
		},

		buildTreeGridTableNode: function __buildTreeGridTableNode(){
			this._gridTable = new com.ibm.team.vvc.server.core.ui.internal.editors.WorkitemLinksTable._grid();
			this._gc.destroys(this._gridTable);
			return this._gridTable.domNode;
		},

		refresh: function __refresh(){

			/*
			 * Steps:
			 * 	1: Authenticate to the friend servers
			 * 		2: Get the work item links
			 * 			3: Process the links and add them to the table
			 */

			var authenticationDeferred = new dojo.Deferred();
			CLMUtils.authenticate(authenticationDeferred);
			authenticationDeferred.then(dojo.hitch(this, function __authenticationSuccess(){
				this._getWorkItemLinks().then(dojo.hitch(this, function __getWorkItemLinksSuccess(results){
					console.log(results);
					this._processWorkItemLinks(results);
				}), dojo.hitch(this, function _getWorkItemLinksFail(error){
					console.error(error);
				}));
			}), dojo.hitch(this, function __authenticationFail(error){
				console.error(error);
			}));
		},

		buildActionsNode: function __buildActionsNode(){
			var actionsNode = this.inherited(arguments);

			var linkToWorkitemButton = VvcButtonFactory.getLinkChangeRequestToChangesetButton({
				uri: this._changesetUri
			}, {
				actionCallback: dojo.hitch(this, function(){
					this.refresh();
				})
			});
			this._gc.destroys(linkToWorkitemButton);
			actionsNode.appendChild(linkToWorkitemButton.domNode);

			return actionsNode;
		},

		_getWorkItemLinks: function __getWorkItemLinks(){

			var queryObj = {
					changeset: this._changesetUri,
					consumerUrl: VvcUtils.getVvcUri()
			};

			var getWorkItemsDef = jazz.client.xhrGet({
				headers: {
					"Content-Type": "application/x-oslc-common-link+xml",
					"Accept": "application/atom+json"
				},
				url: VvcUtils.getVvcUri() + "/changesetLinks?" + dojo.objectToQuery(queryObj),
				handleAs: "json",
			});
			return getWorkItemsDef;
		},

		_processWorkItemLinks: function __processWorkItemLinks(results){
			this._gridTable.refresh();

			if(results.length <= 0){
				// Nothing to do
				return;
			}

			var workitems = []

			for(var i = 0; i < results.length; i++){
				result = results[i];
				target = result.target;
				result.id = parseInt(target.substring(dojo.lastIndexOf(target, '/') + 1)),
				result.changesetUri = this._changesetUri
				workitems.push(result);
			}

			this._gridTable.addRows(this._sortWorkitems(workitems));
		},

		/**
		 * Sorts the work items by their ID, biggest first.
		 * 
		 * @return A sorted list of work items
		 */
		_sortWorkitems: function __sortWorkitems(workitems){
			var sorted = [];
			var unsorted = workitems; // Make a copy to be destructively modified
			var left, right;

			sorted.push(unsorted.pop()); // Warm it up
			while(unsorted.length > 0){
				left = unsorted.pop();
				for(var i = 0; i < sorted.length; i++){
					right = sorted[i];
					if(left.id < right.id){
						// Biggest first
						continue;
					}
					sorted.splice(i, 0, left);
					break;
				}
			}
			return sorted;
		},

	});


	dojo.declare("com.ibm.team.vvc.server.core.ui.internal.editors.WorkitemLinksTable._grid", [com.ibm.team.vvc.server.core.ui.internal.grid.VvcTreeGridTable], {

		registerRowBuilders: function __registerRowBuilders(){
			builders = [];
			builders.push(this._workItemBuilder());
			builders.push(this._actionsBuilders());
			return builders;
		},

		_workItemBuilder: function __workItemBuilder(tr, data, table){
			var builder = {
					columnName: messages.workItemLinksTitle,
					columnClass: "workItemsColumn",
					builderFunc: dojo.hitch(this, function __builderFunc(tr, data, table){
						var readyDeferred = new dojo.Deferred();
						var superReadyDeferred = this._defaultBuilder(tr, data, table);
						dojo.when(superReadyDeferred, dojo.hitch(this, function __superReadyDeferredWhen(td){					
							var gc = tr._gc;

							var linkIconNode = dojo.create("span", {
								innerHTML: "&nbsp;",
							}, td);

							var link = new ResourceLink({
								uri: data.target,
								label: data.id,
								iconNode: linkIconNode 
							});
							td.appendChild(link.domNode);  

							readyDeferred.resolve(td);
						}));
						return readyDeferred;
					})
			};
			return builder;
		},

		_actionsBuilders: function __actionsBuilders(tr, data, table){
			var builder = {
					columnName: messages.actions,
					columnClass: "workItemsColumn",
					builderFunc: dojo.hitch(this, function __builderFunc(tr, data, table){
						var readyDeferred = new dojo.Deferred();
						var superReadyDeferred = this._defaultBuilder(tr, data, table);
						dojo.when(superReadyDeferred, dojo.hitch(this, function __superReadyDeferredWhen(td){					
							var gc = tr._gc;

							var removeLinkButton = VvcButtonFactory.getDeleteChangesetLinkButton(data, {
								actionCallback: dojo.hitch(this, function(){
									table.onRefresh();
								})
							});

							gc.destroys(removeLinkButton);
							td.appendChild(removeLinkButton.domNode);

							readyDeferred.resolve(td);
						}));
						return readyDeferred;
					})
			};
			return builder;
		}

	});

})();
