/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013, 2015.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){

dojo.provide("rmps.app.ui.internal.editors.SelectConfigurationFragment");

dojo.require("rmps.core.util.ContextUtils");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.configuration.web.context.ContextPicker");
var ContextPicker = com.ibm.team.configuration.web.context.ContextPicker;
var ContextPickerConfigurationFilterTypes = com.ibm.team.configuration.web.context.ContextPickerConfigurationFilterTypes;

var ContextUtils = rmps.core.util.ContextUtils;

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.app.ui.internal.editors", "SelectConfigurationFragmentMessages");
var messages= dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "SelectConfigurationFragmentMessages");

dojo.declare("rmps.app.ui.internal.editors.SelectConfigurationFragment", [dijit._Widget, dijit._Templated], {

	templateString: '<div>' +
						'<label style="margin-right:2px;margin-left:5px;font-weight:bold;" dojoAttachPoint="_labelPrefix"></label>' +
						'<a style="margin-right:10px;margin-left:2px;" href="#" dojoAttachPoint="_selectedLabel" dojoAttachEvent="onclick: modify"></a>' +
					'</div>',
					
	//local copy for use in template string
	messages: messages,
	
	//mixin properties
	contextSelected: null, //function; called when a new context is selected from the dialog, or when the context is cleared
	errorCallback: null, //function; called when an error occurred
	projectConfigurationsInfo: null,
	contextUrl: null,
	
	setContext: function(contextUrl) {
		if (!contextUrl) {
			this.updateLabel(undefined, undefined, undefined);
			return;
		}
		
		this.contextUrl = contextUrl;
		//load the context object to get the name
		var getContextDeferred = ContextUtils.getContextDeferred(contextUrl, this.projectConfigurationsInfo.projectId);
		getContextDeferred.addCallback(this, function(contextObject){
			//the context object may be a Stream or a Baseline.  Both objects have a "name" field
			this.updateLabel(contextObject.name, ContextUtils.getIconUrlForContext(contextObject.uri), ContextUtils.isWorkspace(contextObject.uri));
		});
		getContextDeferred.addErrback(this, function(error) {
			if (typeof this.errorCallback == 'function') {
				this.errorCallback(error);
			}
		});
	},
	
	_contextSelected: function(contextObject) {
		this.setContext(contextObject);
		this.contextSelected(contextObject);
	},
	
	/**
	 * Method called when the user clicks on the "select" button - opens picker for selecting the
	 * default context.
	 */				
	modify: function(e) {
		if (e) {
			e.preventDefault();
		}
		
		var picker = new ContextPicker({
			filterObject: {
				validTypes: [ContextPickerConfigurationFilterTypes.streams, 
				             ContextPickerConfigurationFilterTypes.baselines, 
				             ContextPickerConfigurationFilterTypes.globalConfigurations]
			},
			projectAreaUrl : this.projectConfigurationsInfo.projectId
		});
		picker.onOk = dojo.hitch(this, function(configuration) {
			this._contextSelected(configuration.uri);	
		});
	},
	
	clear: function() {
		this._contextSelected(null);
	},
	
	/**
	 * Updates the label with the name of the currently selected context.
	 * labelText - String, the new label text - required
	 * iconUrl - URL string for the icon; may be null
	 */
	updateLabel: function(labelText, iconUrl, isWorkspace) {
		this._selectedLabel.innerHTML = "";
		this._labelPrefix.innerHTML = ""; 
		
		if (iconUrl) {
			var img = dojo.create("img", {src:iconUrl}, this._selectedLabel);
		}
		
		this._selectedLabel.appendChild(dojo.doc.createTextNode(labelText));
		
		if (isWorkspace !== undefined) {
			this._labelPrefix.appendChild(dojo.doc.createTextNode(isWorkspace ? messages.workspacePrefix : messages.snapshotPrefix));			
		}
		
		if (!labelText) {
			this.updateLabel(messages.contextNotSetLabel)
		}
	}

});

})();

