/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2015.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
   
dojo.provide("rmps.app.ui.internal.history.HistoryWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require('dojo.i18n');
dojo.require("jazz.ui.toolbar.Separator");
dojo.require("jazz.ui.Toolbar");
dojo.require("rmps.app.ui.internal.history.HistoryTable");

dojo.require("jazz.app.ua.PageHoverHelp");

dojo.require("jazz.util.ResizeNotifier");

dojo.requireLocalization("rmps.app.ui.internal.history", "HistoryMessages");

(function ()
{
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	var TITLE = "http://purl.org/dc/terms/title";
	
	dojo.declare("rmps.app.ui.internal.history.HistoryWidget", [dijit._Widget, dijit._Templated], {
	  
		widgetsInTemplate: true,
		
		templatePath: dojo.moduleUrl('rmps.app', 'ui/internal/history/templates/HistoryWidget.html'),
	   
		constructor: function() {
			
			this.messages = dojo.i18n.getLocalization("rmps.app.ui.internal.history", "HistoryMessages");
			
			this.explorerUrl = dojo.moduleUrl('rmps.app', 'graphics/icons/history.gif');
			
			this._getConfigurationContext();
			
			dojo.subscribe("history.historyLastestVersionsTitle", dojo.hitch(this, function() 
			{
				this._historyHeading.innerHTML = dojo.string.substitute(this.messages.historyHeading, [ arguments[0], this._configurationContextTitle ]);
			} ));
		},
		
		postCreate: function(){
			var self = this;
			
			this._constructToolbar();
			this._constructTable();
			this._initializeCSH();
		},
		
		_initializeCSH: function() {	
			this._ua = new jazz.app.ua.PageHoverHelp({uaKey: "rmps.app.ui.internal.history.page_HistoryView001"});
			this._pageHeader.appendChild(this._ua.domNode);
			this._ua.domNode.style.margin = "0px 0px 0px 5px";	
	   	},
	   	
		_getConfigurationContext: function() {
			var configurationUri = jazz.app.context.get().baseline;
			var args = {
				url: configurationUri,
				sync: true,
				headers: {
					"Accept": "application/rdf+json"
				},
				handleAs: "json",
				load: function(response, ioArgs) {
					return response[configurationUri];
				},
				error: function(response, ioArgs) {
					this._configurationContextTitle = "";
				}
			};
			this.ConfigurationContextDeffered = jazz.client.xhrGet(args);
			if(this.ConfigurationContextDeffered) {
				this.ConfigurationContextDeffered.addCallback(this, this._getConfigurationContextTitle);
			}
		},
		
		_getConfigurationContextTitle: function(configurationRDF) {
			if (configurationRDF && configurationRDF[TITLE]) {
				this._configurationContextTitle = configurationRDF[TITLE][0].value;
			} else {
				this._configurationContextTitle = "";
			}
		},
		
		_constructToolbar:function()
		{
			if (!this._toolbar)
			{
				var toolbar = new jazz.ui.Toolbar();
				
				var refreshButton = jazz.ui.toolbar.Button({
					title: this.messages.refresh,
					imgSrc: dojo.moduleUrl('rmps.app', 'graphics/icons/refresh.gif'),
					onClick: dojo.hitch(this, function(){
						if (this._table)
							this._table.refresh();
					})
				});
				toolbar.addChild(refreshButton);
				
				this._toolbar = toolbar;
				this._toolbarDiv.appendChild(toolbar.domNode);
			}
		},
		
		_constructTable: function() {
			if (!this._table)
			{
				this._table = new rmps.app.ui.internal.history.HistoryTable({
					"displayErrorMessage" : dojo.hitch(this, this.displayErrorMessage),
					"hideErrorMessage" : dojo.hitch(this, this.hideErrorMessage)
				});
				
			    /* append the new table to the div */
			    this._mainDiv.appendChild(this._table.domNode);
		
			}
		},
	    
	    uninitialize: function()
		{
	    	if (this._table) {
				this._table.destroyRecursive();
				delete this._table;
			}
	    	if (this._toolbar) {
				this._toolbar.destroyRecursive();
				delete this._toolbar;
			}
		},
		
		displayErrorMessage: function(errorMessage) 
		{
			var mb = new jazz.ui.MessageBox();
			mb.placeAt(this._historyErrorMessages);
			mb.showMessage({summary: errorMessage, type: jazz.ui.MessageBox.STATUS_TYPES.ERROR});
			dojo.style(this._historyErrorMessages, {display: ""});
		},
		
		hideErrorMessage: function() {
			if (this._historyErrorMessages) {
				dojo.empty(this._historyErrorMessages);
				dojo.style(this._historyErrorMessages, {display: "none"});
			}
		}
	});   
	})();
})();