/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2015.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {
	dojo.provide("rmps.app.ui.internal.linkvalidity.LinkValidityUtils");
	dojo.require("rmps.core.RootServices");
	dojo.require("rmps.core.util.ContextUtils");
	var ContextUtils = rmps.core.util.ContextUtils;
	
	
	dojo.require("dojo.i18n");
	
	rmps.app.ui.internal.linkvalidity.LinkValidityUtils = {
			
		ISVALID_URI: "/checkValidity/isValid",
		GETINFO_URI: "/checkValidity/getInfo",
		UPDATEVALIDITY_URI: "/updateValidity/update",
		BULKUPDATEVALIDITY_URI: "/updateValidity/bulkUpdate",
		VALIDTARGETVERSIONS_URI: "/checkValidity/validTargetVersions",
		VALIDSOURCEVERSIONS_URI: "/checkValidity/validSourceVersions",
		ROLLUP_URI: "/checkValidity/rollup",
        VALIDITYCONFIG_URI: "/validityConfig",
        IS_VALIDITY_ENABLED: "/checkValidity/isValidityEnabled",
        
        _validityStateCache: {},
		
		isLinkValidityEnabledForProject: function(projectUri) {
			//	Returns true if link validity is enabled for the specified project.
            if (com.ibm.team.web.validity.enabled && ProjectCache.getCmState(projectUri)){
                
                if (this._validityStateCache[projectUri]){
                    return this._validityStateCache[projectUri];
                }
                else{
                    var xhrArgs = {
                        headers: {
                            "Content-Type": "application/json"
                        },
                        handleAs: "json",
                        preventCache: false,
                        sync: true
                    };
                    
                    xhrArgs.headers[Headers.OWNING_CONTEXT] = projectUri;

                    RDMClient.doGet({
                            uri: this.VALIDITYCONFIG_URI + "?projectUri=" + projectUri,
                            xhrArgs: xhrArgs
                        }).then(dojo.hitch(this, function(response) {
                            this._validityStateCache[projectUri] = response.doc.isEnabled;
                    }));
                    return this._validityStateCache[projectUri];
                }
            } else{
                return false;
            }
		},
        
        setLinkValidityEnabledForProject: function(projectUri, enabled) {
            var postData = {
                "isEnabled": enabled
            };
            var xhrArgs = {
                headers: {
                    "Content-Type": "application/json"
                },
                handleAs: "json"
            };
            
            xhrArgs.headers[Headers.OWNING_CONTEXT] = projectUri;
            
            return RDMClient.doPost({
				uri: this.VALIDITYCONFIG_URI,
	            postData: JSON.stringify(postData),
	            xhrArgs: xhrArgs
			}).then(dojo.hitch(this, function(response) {
                var isEnabled = response.isEnabled;
                this._validityStateCache[projectUri] = isEnabled;
                return isEnabled;
            }));
        },
		
		isValid: function(params, configurationUri) {
			// summary:
			//		For each set of params describing a link, returns a boolean value for the validity of the link
			// params: a JSON object or an array of links described as the following
			//	{
	    	//		"linkType": <uri of the link type>,
	    	//		"srcConcept": <concept uri of the source resource for the link>,
	    	//		"targetConcept": <concept uri of the target resource for the link>
	    	//	}
			// configurationUri:
			//	Optional. The configuration uri that will be used to determine the source and target versions for the link.
			//	If not provided, the currently loaded configuration will be used.
			// returns:
			//		A deferred that resolves to a JSON array of boolean values. True if the link is valid.
			
			return this._constructRequest(this.ISVALID_URI, params, configurationUri, true).then(dojo.hitch(this, function(res) {
				return res;
			}));
		},
		
		getInfo: function(params, configurationUri) {
			// summary:
			//		For each set of params describing a link, returns full validity details
			//	{
	    	//		"linkType": <uri of the link type>,
	    	//		"srcConcept": <concept uri of the source resource for the link>,
	    	//		"targetConcept": <concept uri of the target resource for the link>
	    	//	}
			// configurationUri:
			//	Optional. The configuration uri that will be used to determine the source and target versions for the link.
			//	If not provided, the currently loaded configuration will be used.
			// returns:
			//		A deferred that resolves to a JSON array of full validity metadata and status.
			
			return this._constructRequest(this.GETINFO_URI, params, configurationUri).then(dojo.hitch(this, function(res) {
				return JSON.parse(res.ioArgs.xhr.response);
			}));
		},
	
		updateValidity: function(params, configurationUri) {
			// summary:
			//		For each set of params describing a link, set the link validity state
			// params: a JSON object or an array of links described as the following
			//	{
	    	//		"linkType": <uri of the link type>,
	    	//		"srcConcept": <concept uri of the source resource for the link>,
	    	//		"targetConcept": <concept uri of the target resource for the link>,
			//		"status" : <Integer. The new validity status of the link. 0 = valid, 1 = invalid, 2 = unknown>
	    	//	}
			// configurationUri:
			//	Optional. The configuration uri that will be used to determine the source and target versions for the link.
			//	If not provided, the currently loaded configuration will be used.
			// returns:
			//		The request deferred.

			return this._constructRequest(this.UPDATEVALIDITY_URI, params, configurationUri, true);
		},
		
		bulkUpdateValidity: function(params, configurationUri) {
			// summary:
			//		For each set of params describing a link type and concept uris, aupdate all of the
			//		links for the concept URIs of the provided linkType to the newly provided status
			// params: a JSON object or an array of links described as the following
			//	{
			//		status: <the updated status>,
			//		validityDescription: <the updated comment>,
			//		linkType: <link type uri>,
			//		srcConcepts: [<array of the concept uris to update all links of the provided 'linkType' to the new 'status'>],
			//		targetConcepts: [<array of the concept uris to update all links of the provided 'linkType' to the new 'status'>]
			//	}
			// configurationUri:
			//	Optional. The configuration uri that will be used to determine the source and target versions for the link.
			//	If not provided, the currently loaded configuration will be used.
			// returns:
			//		The request deferred.
			
			return this._constructRequest(this.BULKUPDATEVALIDITY_URI, params, configurationUri);
		},
		
		getValidTargetVersions: function(params, configurationUri) {
			// summary:
			//		For each set of params describing a link, returns a the last valid version of the target
			// params: a JSON object or an array of links described as the following
			//	{
	    	//		"linkType": <uri of the link type>,
	    	//		"srcConcept": <concept uri of the source resource for the link>,
	    	//		"targetConcept": <concept uri of the target resource for the link>
	    	//	}
			// configurationUri:
			//	Optional. The configuration uri that will be used to determine the source and target versions for the link.
			//	If not provided, the currently loaded configuration will be used.
			// returns:
			//		A deferred that resolves to a JSON array valid target versions.
			
			return this._constructRequest(this.VALIDTARGETVERSIONS_URI, params, configurationUri).then(dojo.hitch(this, function(res) {
				return JSON.parse(res.ioArgs.xhr.response);
			}));
		},
		
		getValidSourceVersions: function(params, configurationUri) {
			// summary:
			//		For each set of params describing a link, returns a the last valid version of the source
			// params: a JSON object or an array of links described as the following
			//	{
	    	//		"linkType": <uri of the link type>,
	    	//		"srcConcept": <concept uri of the source resource for the link>,
	    	//		"targetConcept": <concept uri of the target resource for the link>
	    	//	}
			// configurationUri:
			//	Optional. The configuration uri that will be used to determine the source and target versions for the link.
			//	If not provided, the currently loaded configuration will be used.
			// returns:
			//		A deferred that resolves to a JSON array valid source versions.
			
			return this._constructRequest(this.VALIDSOURCEVERSIONS_URI, params, configurationUri).then(dojo.hitch(this, function(res) {
				return JSON.parse(res.ioArgs.xhr.response);
			}));
		},
		
		getValidityRollupData: function(params, getDetails, configurationUri) {
			// summary:
			//		For each provided conceptUri, returns details and summary of the validity statuses for the links
			// params: 
			//		an array of concept Uris
			//	getDetails:
			//		boolean. if set, will retrieve the detailed roll-up data as opposed to just the summary data.
			// configurationUri:
			//	Optional. The configuration uri that will be used to determine the source and target versions for the link.
			//	If not provided, the currently loaded configuration will be used.
			// returns:
			//		A deferred that resolves to an array of roll-up information for the provided concept Uris.
			return this._constructRequest(this.ROLLUP_URI + (getDetails ? "?validityDetails=true" : ""), params, configurationUri).then(dojo.hitch(this, function(res) {
				return JSON.parse(res.ioArgs.xhr.response);
			}));
		},
		
		isValidityEnabled: function(projectId) {
			var retval = false;
			var params = {
				projectId : projectId	
			};
			
			var def = jazz.client.xhrGet({
				sync: true,
				url : net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + this.IS_VALIDITY_ENABLED + "?" + dojo.objectToQuery(params)
			}).then(dojo.hitch(this, function(result){
				if(result === "true") {
					retval = true;
				}
				return retval;
			}), dojo.hitch(this, function(error){
				console.error(error);
				retval = false;
				return retval;
			}));
			return retval;
		},
		
		getLinkValidityFormatFromLinkData: function(linkData) {
			if (linkData) {
				if (linkData.subject) {
					return {
	                    linkType: linkData.predicate,
	                    linkUri: linkData.uri,
	                    sourceConceptId: linkData.subject,
	                    sourceConceptVersion: linkData.subject,
	                    targetConceptId: linkData.value,
	                    targetConceptVersion: linkData.value,
	                    status: linkData.status
					};
				} else {
					return {
	                    linkType: linkData.predicate,
	                    linkUri: linkData.uri,
	                    sourceConceptId: linkData.value,
	                    sourceConceptVersion: linkData.value,
	                    targetConceptId: linkData.resource,
	                    targetConceptVersion: linkData.resource,
	                    status: linkData.status
					};
				}
			}
			return null;
		},
		
		_constructRequest: function(path, params, configurationUri) {
			return this._constructRequest(path, params, configurationUri, false);
		},
		
		_constructRequest: function(path, params, configurationUri, sync) {
			var globalConfigurationUri = ContextUtils.getExpandedContext();
			var contextUri = (globalConfigurationUri) ? globalConfigurationUri : ContextUtils.getCurrentContextURI();
			url = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + path + "?oslc_config.context=" + encodeURIComponent(contextUri) + "&projectId=" + jazz.app.context.get().itemId;
			var postData = [];
			if (params && !params.length) {
				postData.push(params);
			} else {
				postData = params;
			}
			var def = jazz.client.xhrPost({
				handleAs: "json",
				sync: sync,
				headers: {
					"Content-Type" : "application/json"
				},
				postData: JSON.stringify(postData),
				url: url,
				error: function(response, ioArgs) {
					rmps.core.error.createXhrError(ioArgs).dispatch();
				}
			});
			return def;
		},
		
		checkUnderEdit: function(ifNotUnderEdit) {
			//  summary:
			//		This method will check to see if there are active edits on the page.
			//		If so, display a dialog stating that changes need to be saved before
			//		updating the validity status.
			//	ifNotUnderEdit:
			//		function to be called if nothing is under edit.
			
			var underEdit =  rmps.app.ui.internal.project.EditManager.isAnyUnderEdit(rmps.app.ui.internal.CURRENT_PROJECT) || dojo.query(".active-edit").length;
			if (underEdit) {
				// Something on the page is under edit. Grab the required strings, then create the dialog. Do not execute the ifNotUnderEdit function.
				var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.linkvalidity", "LinkValidityControlWidget");
				var content = dojo.create("div");
				content.appendChild(document.createTextNode(messages.unsavedChangesDialogMessage));
				var msgForm = new MessageForm({
					confirmContent: content,
					dialogTitle: messages.unsavedChangesDialogTitle,
					dialogWidth: "350px",
					dialogStyle: "ok"
				});
				msgForm.show();
			} else if (ifNotUnderEdit) {
				ifNotUnderEdit.call();
			}
		},
		
		getHelpTopicUri: function() {
			//	summary:
			//		Get the URI to the link validity help topic. Unfortunately, this requires two calls
			//		in order to retrieve the full URI.. One for the prefix, then one for the content which is then
			//		used to retrieve the other parts that must be parsed from the actual help content.
			//		If this seems crazy, that is because it is. However, it is unavoidable with the
			//		current way the jazz ua code works.
			//	returns:
			//		a deferred that resolves to the link validity help topic URI
			
			var uaKey = rmps.app.ui.internal.help.context.linkValidity,
				fakeDeferred = new dojo.Deferred();
			
			var helpPrefixDeferred = jazz.app.ua.internal.client.getTopicPrefix(uaKey),
				helpContentDeferred = jazz.app.ua.internal.client.getContent(uaKey);
			var deferredList = new dojo.DeferredList([helpPrefixDeferred, helpContentDeferred]);
			deferredList.then(function(results){
				if (results.length === 2 && results[0][1] && results[1][1]) {
					try {
						var prefix = results[0][1],
							doc = dojox.xml.parser.parse(results[1][1]);
						var topic = dojo.query("topic", doc)[0];
						if (topic) {
							var href = dojo.attr(topic, "href");
							if (href) {
								fakeDeferred.callback(prefix + href);
							}
						}
					} catch(e) {
						console.error(e);
						fakeDeferred.callback(null);
					}
				} else {
					fakeDeferred.callback(null);
				}
			});
			return fakeDeferred;
		}
		
	}; // End LinkValidityUtils
	
	rmps.app.ui.internal.linkvalidity.LinkValidityUtils.ENDPOINTS = {
		source: "source",
		target: "target",
		sourceOrTarget: "sourceOrTarget"
	};
	
	rmps.app.ui.internal.linkvalidity.LinkValidityUtils.EVENT_SOURCE = {
		grid: "grid",
		gridLinksColumn: "gridLinksColumn",
		savedFilter: "savedFilter",
		viewlet: "viewlet",
		overview: "overview",
		linksSidebar: "linksSidebar",
		artifactToolbar: "artifactToolbar"
	};
	
	rmps.app.ui.internal.linkvalidity.LinkValidityUtils.LINK_STATUS = {
		IGNORED: -1,
		VALID: 1,
		INVALID: 2,
		UNKNOWN: 3,
		DO_NOT_CHANGE: "DO_NOT_CHANGE" // A psuedo-status used only within the web client for when we do not want to alter the validity state in the bulk update widget.
	};
})();