/*
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM.  Reproduction, modification and redistribution are prohibited.
 * 
 * (c) Copyright IBM Corporation 2015.  
 * U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */
(function() {
    dojo.provide("rmps.app.ui.internal.pages.DmStreamEditor");
    
    dojo.require("jazz.ui.toolbar.Button");
    var Button = jazz.ui.toolbar.Button;
    
    dojo.require("rmps.app.ui.internal.pages.WorkspacesAndSnapshotsViewer");
    var WorkspacesAndSnapshotsViewer = rmps.app.ui.internal.pages.WorkspacesAndSnapshotsViewer;
    
    dojo.require("rmps.app.ui.internal.util.ConfigurationResource");
    var ConfigurationResource = rmps.app.ui.internal.util.ConfigurationResource;
    
    dojo.require("com.ibm.team.repository.web.configurationManagement.editors.CmTabManager");
    var Tab = com.ibm.team.repository.web.configurationManagement.editors.CmTab;
    
    dojo.require("rmps.app.ui.internal.changesets.DmChangeSetsExplorer");
    var DmChangeSetsExplorer = rmps.app.ui.internal.changesets.DmChangeSetsExplorer;
    
    dojo.requireLocalization("rmps.app.ui.internal.pages", "DmStreamEditorMessages");
    var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages", "DmStreamEditorMessages");
    
    dojo.requireLocalization("com.ibm.team.repository.web.configurationManagement.editors", "CmEditorMessages");
    var CmMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.configurationManagement.editors", "CmEditorMessages");
    
    dojo.require("rmps.app.ui.internal.util.Constants");
    var Constants = rmps.app.ui.internal.util.Constants;
    
    dojo.require("com.ibm.team.configuration.web.context.ContextPicker");
    var ContextPicker = com.ibm.team.configuration.web.context.ContextPicker;
    var ContextPickerConfigurationFilterTypes = com.ibm.team.configuration.web.context.ContextPickerConfigurationFilterTypes;

    var DEPENDENCY_SERVICE = "/dependencies";
    
    dojo.require("com.ibm.team.repository.web.configurationManagement.editors.CmStreamEditor");
//    dojo.require("com.ibm.rdm.web.pages._RmConfigurationEditorMixin");
    var CmStreamEditor = com.ibm.team.repository.web.configurationManagement.editors.CmStreamEditor;
  //  var _RmConfigurationEditorMixin = com.ibm.rdm.web.pages._RmConfigurationEditorMixin;
   
    // Summary:
    //      This is a simple subclass of the foundation Stream editor. Just adds the project name
    //      and context button to the banner.
    dojo.declare("rmps.app.ui.internal.pages.DmStreamEditor", 
                    [CmStreamEditor], {

         // CmEditor will register this Action with the ActionRegistry...needs to match
         // the entry in com.ibm.rdm.web/plugin.xml                     
         editorAction: "rmps.app.editStream",
         workspacesViewer: null,
         
		_buildTabs: function __buildTabs(){
			var tabs = this.inherited(arguments);
			tabs.push(this.buildDependenciesTab());
			return tabs;
		},
		
		//override the buildChangesetsTab in com.ibm.team.repository.web.configurationManagement.editors.CmStreamEditor
		buildChangesetsTab: function __buildChangesetsTab(){	
			var initialiser = dojo.hitch(this, function(){
				var changesetsExplorer = new DmChangeSetsExplorer({
					configurationUri: this.resourceUri
				});
				return changesetsExplorer;
			});
			
			var tab = new Tab({
				id: "changesets",
				title: CmMessages.changesetsTabTitle,
				helpContent: CmMessages.changesetsTabHelpContent,
				index: 0,
				initialiser: initialiser
			});
			return tab;
		},
		
		buildDependenciesTab: function __buildWorkspacesTab(){
			var initialiser = dojo.hitch(this, function(){
				// (amz) This whole thing is a 'tactical fix'
				var workspaceResource = new ConfigurationResource();
				workspaceResource.load(this.resourceUri);
				
				var containerNode = dojo.create("div",{
					style: "width: 99%;"
				});
				
				var actionsNode = dojo.create("div", {
					"class": "dependencyActions"
				},containerNode);
				
				var addDependencyButton = new Button({
					title: messages.addDependency,
					imgClass: "addDependencyButtonIcon",
					onClick: dojo.hitch(this, function __onClickAddDependencyButton(){
						
						var picker = new ContextPicker({
							filterObject: {
								validTypes: [ContextPickerConfigurationFilterTypes.baselines]
							},
							projectAreaUrl: Constants.NONE
						});
						picker.onOk = dojo.hitch(this, function(configuration) {
							this._addSelected(configuration.uri);
						});
					})
				});
				actionsNode.appendChild(addDependencyButton.domNode);
				
				this.workspacesViewer = new WorkspacesAndSnapshotsViewer({
					workspace: workspaceResource,
					workspaceUri: this.resourceUri,
					style: 2, 
					lazyStart: false
				});
				containerNode.appendChild(this.workspacesViewer.domNode);
				
				return containerNode;
			});
			
			var tab = new Tab({
				id: "dependencies",
				title: messages.dependenciesTabTitle,
				index: 1,
				initialiser: initialiser,
				helpContent: messages.dependenciesHelpContent
			});
			return tab;
		},
		
		_addSelected: function(dependencyUri) {
			if (dependencyUri) {
				this.workspacesViewer.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
					summary: messages.adding
				});
				
				var args = {};
				args.configurationUri = this.resourceUri;
				args.dependencyUri = dependencyUri;
				uri = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + DEPENDENCY_SERVICE + "/addDependency?" + dojo.objectToQuery(args);
				var self = this;
				var args = {
					url: uri,
					headers: {
						"Accept" : "application/json"
					},
					sync: true,
					handleAs: "json",
					error: dojo.hitch(this, function(e, ioArgs) {
						this.workspacesViewer.hideMessage();
						rmps.core.error.createXhrError(ioArgs).dispatch();
					})
				};
				var deferred = jazz.client.xhrPost(args);
				deferred.addCallback(this, function (param) {
					this.workspacesViewer.hideMessage();
					this.workspacesViewer.refresh();
				});
			} else {
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: this.messages.addEmptySelection
				});
			}
		}
    });
}());