/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corporation 2014. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("rmps.app.ui.internal.util.ConfigurationUiNode");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojox.html.entities");
dojo.require("dojo/on");

dojo.require("jazz.ui.HoverPopup");
var HoverPopup = jazz.ui.HoverPopup;

dojo.require("rmps.core.util.ContextUtils");
var ContextUtils= rmps.core.util.ContextUtils;

dojo.require("jazz.ui.ResourceLink");
var ResourceLink = jazz.ui.ResourceLink;

dojo.require("rmps.app.ui.internal.gc.DmGarbageCollector");
var GC = rmps.app.ui.internal.gc.DmGarbageCollector;

dojo.requireLocalization("rmps.app.ui.internal.util", "ConfigurationUiNodeMessages");

dojo.declare("rmps.app.ui.internal.util.ConfigurationUiNode", [dijit._Widget], {
	
    messages: dojo.i18n.getLocalization("rmps.app.ui.internal.util", "ConfigurationUiNodeMessages"),
	
	_uri: null,
	_includeLink: null,
	_includeHover: null,
	_isTitle: null,
	_parentDomNode: null,
	_highlightSequences: null,
	_gc: null,
	
    constructor: function __ConfigurationUiNode(args, containerNode){
    	if(!args){
    		throw "ConfigurationUiNode missing arguments!"
    	}
    	this._gc = new GC();
    	if (args.configResource) {
    		this._configResource = args.configResource;
    		this._uri = this._configResource.uri;
    	} else {
    		this._uri = args.uri;
    	}

    	this._includeLink = args.includeLink !== undefined ? args.includeLink : true;  	    	
    	this._includeHover = args.includeHover !== undefined ? args.includeHover : true;
    	this._isTitle = args.isTitle !== undefined ? args.isTitle: false;
    	this._highlightCharacters = args.highlightCharacters !== undefined ? args.highlightCharacters : [];
    	
    	if(containerNode){
    		this._parentDomNode = containerNode;
    	}
    },
    
    destroy: function __destroy(){
    	this.inherited(arguments);
    	this._parentDomNode = null;
    	this._uri =  null;
    	this._includeLink = null;
    	this._includeHover = null;
    	this._isTitle = null;
    	this._parentDomNode = null;
    	this._highlightSequences = null;
    	this.domNode = null;
    	this.messages = null;
    	this._removeStopClick();
    	this._gc.destroy();
    	this._gc = null;
    },
    
    postCreate: function __postCreate(){
    	this.domNode = dojo.create("a", {
    		title: this.messages.loading,
    		href: "#",	// Link to nowhere
    		"class": "configurationUiNode"
    	});
    	
    	if(this._isTitle){
    		dojo.addClass(this.domNode, "titleNode");
    	}
    	
    	this.refresh();
    	
    	if(this._parentDomNode){
    		this._parentDomNode.appendChild(this.domNode);
    	}
    	this._gc.destroys(this.domNode);
    },
        
    refresh: function __refresh(){	
    	var loadingNode = this._buildLoadingNode();
    	dojo.empty(this.domNode);
    	this.domNode.appendChild(loadingNode);
    	dojo.setAttr(this.domNode, "disabled", "true");
    	
    	// Clear any existing stop clicks, then set a stop-click while the node loads.
    	this._removeStopClick();
		this._setStopClick();
    	
    	dojo.when(this._getContent(), dojo.hitch(this, function ___getContent(){
    		if(this._includeLink){
        		dojo.removeAttr(this.domNode, "disabled");
        		this._removeStopClick();
    		}
    	}), dojo.hitch(this, function __errorHandler(error){
    		dojo.removeAttr(this.domNode, "disabled");
    		this._buildErrorNode(error);
    		dojo.destroy(loadingNode);
    	}));
    },
    /**
	  * Prevents the user from clicking on the DOM node.
	  */
    _setStopClick: function(){
        var setStopClick = dojo.connect(this.domNode, "onclick", dojo.hitch(this, function __setStopClick(event){
    		dojo.stopEvent(event); // Stop the DOM events from firing (page redirect)
    		if(this.onclick){
    			this.onclick(); // Faux click event
    		}
    	}));
    	
    	var removeStopClick = dojo.connect(this, "_removeStopClick", dojo.hitch(this, function __removeStopClick(){
			dojo.disconnect(setStopClick);
			dojo.disconnect(removeStopClick);
    	}));
    	return setStopClick;
    },
    
    _removeStopClick: function __removeStopClick(){
    	// Stub
    },
        
    _buildLoadingNode: function __buildLoadingNode(){
    	var loadingNode = dojo.create("span");	
    	
    	var iconNode = dojo.create("span", {
    		"class": "iconNode"
    	});
    	this._gc.destroys(iconNode);
    	
    	var icon = dojo.create("img", {
    		src: dojo.moduleUrl('rmps.app', 'graphics/icons/spinner.gif'),
			alt: this.messages.loading
    	});
    	iconNode.appendChild(icon);
    	this._gc.destroys(icon);
    	
    	var titleNode = dojo.create("span", {
    		"class": "titleNode",
    		innerHTML: this.messages.loading
    	});
    	this._gc.destroys(titleNode);
    	
    	loadingNode.appendChild(iconNode);
    	loadingNode.appendChild(titleNode);
    	
    	this._gc.destroys(loadingNode);
    	return loadingNode;
    },
    
    _buildErrorNode: function __buildErrorNode(error){
    	dojo.empty(this.domNode);
		dojo.removeAttr(this.domNode, "title");
	
    	var contentNode = dojo.create("span", {
    		"class": "error"
    	});
    	this._gc.destroys(contentNode);
		
    	var iconNode = dojo.create("span", {
    		"class": "iconNode"
    	});
    	this._gc.destroys(iconNode);
    	
    	var titleNode = dojo.create("span", {
    		"class": "titleNode",
    		innerHTML: this.messages.error
    	});
    	this._gc.destroys(titleNode);
    	
    	contentNode.appendChild(iconNode);
    	contentNode.appendChild(titleNode);
    			
		var onClickHandler = dojo.connect(this.domNode, "onclick", this, function __onClickHandler(event){
			dojo.stopEvent(event);
			dojo.disconnect(onClickHandler);
			this.refresh();
		});
		this._gc.disconnects(onClickHandler);
		
		var messageNode = dojo.create("div", {
			innerHTML: this.messages.clickHereToRefreshNodeTitle
		});
		this._gc.destroys(messageNode);
    	
    	var errorPopup = new HoverPopup({
    		around: contentNode,
    		createContents: dojo.hitch(this, function __createMessageNode(){
    			return messageNode;
    		})
    	});
    	this._gc.destroys(errorPopup);

		this.domNode.appendChild(contentNode);
    },
        
    _getContent: function __getContent(){
    	var deferred = ContextUtils.getContextDeferred(this._uri);
  	
    	dojo.when(deferred, dojo.hitch(this, function __deferredThen(conf){
    		var contentNode = this._buildContentNode(conf);
    		dojo.empty(this.domNode);
    		dojo.removeAttr(this.domNode, "title");
    		
    		if(conf.archive){
    			dojo.addClass(this.domNode, "archived");
    			this._includeLink = false;
    		}
    		
    		if(this._includeLink){
    			dojo.attr(this.domNode, "href", conf.uri);
    		}
    		this.domNode.appendChild(contentNode);
    	}));
    	return deferred;
    },
    
    _buildContentNode: function __buildContentNode(conf){
    	var type = ContextUtils.getContextType(conf.uri);
    	var contentNode = dojo.create("span", {
    		"class": type.value.toLowerCase()
    	});
    	this._gc.destroys(contentNode);
    	
    	var iconNode = dojo.create("span", {
    		"class": "iconNode"
    	});
    	this._gc.destroys(iconNode);
    	
    	var titleNode = dojo.create("span", {
    		"class": "titleNode",
    		innerHTML: this.buildTitleNode(conf.name)
    	});
    	this._gc.destroys(titleNode);
    	
    	if(conf.archive){
    		this._includeHover = false;
    	}
    	
    	if(this._includeHover){
    		var resourceLink = this.getHover(conf, this.domNode);
    	}
    	
    	contentNode.appendChild(iconNode);
    	contentNode.appendChild(titleNode);
    	
    	return contentNode;
    },
    
    getHover: function __getHover(conf, titleNode){
    	var resourceLink = new ResourceLink({
			uri: conf.uri,
			label: conf.name,
			around: titleNode,
			lazyFetch: true
    	});
    	this._gc.destroys(resourceLink);
    	return resourceLink;
    },
    
    buildTitleNode: function __buildTitleNode(title){
    	if(!title || title.length === 0) {
    		return "";
    	}
    	
    	var newTitle = title;
    	if(this._highlightCharacters.length > 0){
    		var start = title.toLocaleLowerCase().indexOf(this._highlightCharacters.toLocaleLowerCase());
    		var end = start + this._highlightCharacters.length;
    		
    		// start < 0 means the string was not found.
    		if(start >= 0){
    			newTitle = title.substring(0, start);
    			newTitle += "<b>" + title.substring(start, end) + "</b>";
    			newTitle += this.buildTitleNode(title.substring(end));
    		}
    	}
    	return newTitle;
    }
});
   
})();
