/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2015.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function DmButtonFactory() {
dojo.provide("rmps.app.ui.internal.util.DmButtonFactory");

dojo.require("jazz.ui.toolbar.Button");
dojo.require("rmps.core.RootServices");

dojo.require("jazz.ui.menu.MenuItem");
var MenuItem = jazz.ui.menu.MenuItem;

dojo.require("rmps.app.ui.internal.util.InfoContentUtils");
var InfoContentUtils = rmps.app.ui.internal.util.InfoContentUtils;

dojo.require("com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.LinkToWorkItemUtil");
var LinkToWorkItemUtil = com.ibm.xtools.rmps.web.deliver.shared.web.ui.util.LinkToWorkItemUtil;

dojo.require("com.ibm.team.repository.web.configurationManagement.notifications.CmNotificationFactory");
var CmNotificationFactory = com.ibm.team.repository.web.configurationManagement.notifications.CmNotificationFactory;

dojo.requireLocalization("rmps.app.ui.internal.pages", "DmChangeSetEditorMessages");
var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages", "DmChangeSetEditorMessages");

dojo.requireLocalization("com.ibm.team.repository.web.configurationManagement", "CmButtonFactoryMessages");
var CmButtonFactoryMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.configurationManagement", "CmButtonFactoryMessages");

rmps.app.ui.internal.util.DmButtonFactory = {
	
	getLinkToWorkItemButton: function (data, addLinkCallback, isMenuItem) {
		var args = {
				imgClass: "icons-changeset-workitem",
				title: messages.addLink,
				onClick: dojo.hitch(this, function() {
					 LinkToWorkItemUtil.linkToWorkItemAction(data, addLinkCallback);
				})
			};
		var applicable = function(button){
			return true;
		};
		return isMenuItem ? this._makeMenuItem(args, null, applicable) : this._makeButton(args, null, applicable);
	},
	
	getReverseChangesetButton: function getReverseChangesetButton(data, options, onClickArgs){
		var args = {
			imgClass: "icons-changeset-reverse",
			title: CmButtonFactoryMessages.reverseChangeset,
			onClick: dojo.hitch(this, function(){							
				InfoContentUtils.showQuestionDialog(
					CmButtonFactoryMessages.reverseChangesetTitle,
					dojo.string.substitute(CmButtonFactoryMessages.reverseChangesetContent, {changesetTitle: data.name}),
					CmButtonFactoryMessages.reverseChangesetOkButton,
					CmButtonFactoryMessages.cancelButton,
					dojo.hitch(this, function(){
						var params = {
							reverse : true,
							streamUri : data.parentStreamUri,
							changesetUri : data.id,
							changesetTitle : data.name 
						};
						var def = jazz.client.xhrPost({
							handleAs: 'text',
							headers: {
								"Content-Type" : "text/plain"
							},
							url: rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "dm-changesets") + "?" + dojo.objectToQuery(params)
						});
						
						var tempContext = {
							closeEvent: dojo.hitch(this, function(){
								if(options && options.actionCallback){
									options.actionCallback();
								}
							})
						};
						var notification = CmNotificationFactory.createInfo({
							title: CmButtonFactoryMessages.initializingReverseChangeset,
							closeOnEvent: {
								context: tempContext,
								event: "closeEvent"
							}
						}).dispatch();
						
						def.then(dojo.hitch(this, function(results){
							tempContext.closeEvent();
						}), dojo.hitch(this, function(error){
							notification.close();
							CmNotificationFactory.createXhrError({xhrError: error, ioArgs: error.response.options.ioArgs}).dispatch();
						}));
					})
				);
			})
		};
		
		var applicable = function(button){
			return "committed" === data.state.toLowerCase();
		};
		
		var button = this._makeMenuItem(args, options, applicable);
		return button;
	},
	
	
	//Helper functions coppied from com.ibm.team.repository.web.configurationManagement.CmButtonFactory
	_makeButton: function (args, options, applicable){
		// Force enabled to be false, wait for the isApplicable code to run
		args.enabled = false;
		if(options && options.titleAsLabel){
			args.label = args.title;
		}
		var button = new jazz.ui.toolbar.Button(args);
		var isApplicable = dojo.hitch(this, applicable, button);
		dojo.when(isApplicable(), dojo.hitch(this, function(isEnabled){
			// Respect the explicit disablement of a button, then respect the buttons own enablement.
			button.set("enabled", options && options.disabled !== undefined ? options.disabled === false : true && isEnabled);
		}));
		return button;
	},
		
	_makeMenuItem: function (args, options, applicable){
		// Make sure the label exists
		args.disabled = true;
		if(args.title && !args.label){
			args.label = args.title;
		}
		// Map the Button 'imgClass' to the Menu Item 'iconClass'
		if(args.imgClass && !args.iconClass){
			args.iconClass = args.imgClass + " vvcIcon";
		}
		
		var menuItem = new MenuItem(args);
		
		var isApplicable = dojo.hitch(this, applicable, menuItem);
		dojo.when(isApplicable(), dojo.hitch(this, function(isEnabled){
			// Respect the explicit disablement of a button, then respect the buttons own enablement.
			var enabled = options && options.disabled !== undefined ? options.disabled === false : true && isEnabled;
			menuItem.set("disabled", !enabled);
		}));
		return menuItem;
	}
};
}());