/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corporation 2015. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function UiDefinitionsNode(){
dojo.provide("rmps.app.ui.internal.util.UiDefinitionNode");

dojo.require("dijit._Widget");

dojo.require("jazz.ui.HoverPopup");
var HoverPopup = jazz.ui.HoverPopup;

dojo.require("jazz.ui.Dialog");
var Dialog = jazz.ui.Dialog;

dojo.require("rmps.app.ui.internal.gc.DmGarbageCollector");
var GC = rmps.app.ui.internal.gc.DmGarbageCollector;

dojo.requireLocalization("rmps.app.ui.internal.util", "UiDefinitionNodeMessages");
var Messages = dojo.i18n.getLocalization("rmps.app.ui.internal.util", "UiDefinitionNodeMessages");

dojo.declare("rmps.app.ui.internal.util.UiDefinitionNode", [dijit._Widget], {
   
	// Protected
	_content: null,
	_titleText: null,
	_definition: null,
	_dialogArgs: null,
	_gc: null,

	/**
	 * Creates a UI definitions node.  Either args.content or
	 * args.title are required.  If both are provided, then
	 * args.content will be used.
	 * 
	 * @param args.title (required) Title text.
	 * @param args.definition (required) The definition to expose.
	 */
	constructor: function __UiDefinitionNode(args, parentDomNode){
		if(args){
			this._titleText = args.titleText;
			this._definition = args.definition;
		}
		this._gc = new GC();
		
		if(parentDomNode){
			var fireOnce = dojo.connect(this, "postCreate", function __attachToParentNode(){
				dojo.disconnect(fireOnce);
				parentDomNode.appendChild(this.domNode);
			});
		}
	},
	
	destroy: function __destroy(){
		this._content = null;
		this._definition = null;
		this._dialogArgs = null;
		this._gc.destroy();
		this._gc = null;
	},
	
	postCreate: function __postCreate(){
		this.domNode = dojo.create("span", {
			"class": "uiDefinitionsNode",
		});
		var contentNode = dojo.create("a", {
			onclick: dojo.hitch(this, function __onClick(event){
				dojo.stopEvent(event);
				hover.open();
			}),
			href: "#",
		}, this.domNode);
		this._gc.destroys(contentNode);
		
		var titleNode = dojo.create("span", {
			innerHTML: this._titleText
		}, contentNode);
		
		var hover = new HoverPopup({
			around: contentNode,
			createContents: dojo.hitch(this, function __createContents(){
				return dojo.doc.createTextNode(this._definition);
			}),
		});
		this._gc.destroys(hover);
	}
});

dojo.declare("rmps.app.ui.internal.util.WorkspaceDefinitionNode", [rmps.app.ui.internal.util.UiDefinitionNode], {
	constructor: function __workspaceDefinitionNode(args){
		this._titleText = Messages.workspaceTitle;
		this._definition = Messages.workspaceDefinition;
	}
});

dojo.declare("rmps.app.ui.internal.util.SnapshotDefinitionNode", [rmps.app.ui.internal.util.UiDefinitionNode], {
	constructor: function __snapshotDefinitionNode(){
		this._titleText = Messages.snapshotTitle;
		this._definition = Messages.snapshotDefinition;
	}	
});

dojo.declare("rmps.app.ui.internal.util.ChangesetDefinitionNode", [rmps.app.ui.internal.util.UiDefinitionNode], {
	constructor: function __changesetDefinitionNode(){
		this._titleText = Messages.changeSetTitle;
		this._definition = Messages.changesetDefinition;
	}
});

dojo.declare("rmps.app.ui.internal.util.ChangesetDefinitionNode.active", [rmps.app.ui.internal.util.UiDefinitionNode], {
	constructor: function __activeChangesetDefinitionNode(args){
		this._titleText = Messages.activeChangeSetTitle;
		this._definition = Messages.activeChangeSetDescription;
	}
});

dojo.declare("rmps.app.ui.internal.util.ChangesetDefinitionNode.complete", [rmps.app.ui.internal.util.UiDefinitionNode], {
	constructor: function __completeChangesetDefinitionNode(){
		this._titleText = Messages.completeChangeSetTitle;
		this._definition = Messages.completeChangeSetDescription;
	}
});

dojo.declare("rmps.app.ui.internal.util.ChangesetDefinitionNode.shared", [rmps.app.ui.internal.util.UiDefinitionNode], {
	constructor: function __sharedChangesetDefinitionNode(){
		this._titleText = Messages.sharedChangeSetTitle;
		this._definition = Messages.sharedChangeSetDescription;
	}
});

dojo.declare("rmps.app.ui.internal.util.GlobalConfigurationDefinitionNode", [rmps.app.ui.internal.util.UiDefinitionNode], {
	constructor: function __globalConfigurationDefinitionNode(){
		this._titleText = Messages.globalConfigurationTitle;
		this._definition = Messages.globalConfigurationDescription;
	}
});

dojo.declare("rmps.app.ui.internal.util.GlobalConfigurationDefinitionNode.mutable", [rmps.app.ui.internal.util.UiDefinitionNode], {
	constructor: function __mutableGlobalConfigurationDefinitionNode(){
		this._titleText = Messages.mutableGlobalConfigurationTitle;
		this._definition = Messages.mutableGlobalConfigurationDescription;
	}
});

dojo.declare("rmps.app.ui.internal.util.GlobalConfigurationDefinitionNode.immutable", [rmps.app.ui.internal.util.UiDefinitionNode], {
	constructor: function __immutableGlobalConfigurationDefinitionNode(){
		this._titleText = Messages.immutableGlobalConfigurationTitle;
		this._definition = Messages.immutableGlobalConfigurationDescription;
	}
});

})();
