/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2015.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {	
	dojo.provide("rmps.core.util.ContextUtils");

	dojo.require("rmps.core.baseline.BaselineClient");
	dojo.require("rmps.core.streams.StreamClient");
	dojo.require("rmps.core.changesets.ChangesetClient");
	dojo.require("rmps.core.error");
	dojo.require("rmps.core.util.IRmpsConstants");
	dojo.require("rmps.app.ui.internal.common.ImageService");
	dojo.require("rmps.core.RootServices");

	dojo.require("jazz.client.xhr");
	dojo.require("rmps.core.util.xhrUtil");

	dojo.require("com.ibm.team.configuration.web.context.ContextDisplayMessagesUtil");
	var ContextDisplayMessagesUtil = com.ibm.team.configuration.web.context.ContextDisplayMessagesUtil;
	
	var PROJECT_AREA_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.JP_0_6, "projectAreas") + '/';
	var COOKIE_NAME = "DM_RECENT_CONTEXTS";
	var RECENTLY_VIEWED_LIMIT = 5;
	var _rootWorkspace = null;
	var ImageService = rmps.app.ui.internal.common.ImageService;
	var xhrUtil= rmps.core.util.xhrUtil;

	rmps.core.util.ContextTypes = new rmps.core.Enum({
		// TYPE: Common reference, uriIdentifier: path in the URI
		WORKSPACE: new rmps.core.EnumLiteral({value: "WORKSPACE", uriIdentifier: "stream", icon: ImageService.WORKSPACE_ICON}),
		SNAPSHOT: new rmps.core.EnumLiteral({value: "SNAPSHOT", uriIdentifier: "baseline", icon: ImageService.BASELINE_EDITOR_ICON}),
		CHANGESET: new rmps.core.EnumLiteral({value: "CHANGESET", uriIdentifier: "changeset", icon: null}),
	});

	var CONTEXT_TYPES = rmps.core.util.ContextTypes;
	var IRmpsConstants = rmps.core.util.IRmpsConstants;
	var StreamClient = rmps.core.streams.StreamClient;
	var BaselineClient = rmps.core.baseline.BaselineClient;
	var ChangesetClient = rmps.core.changesets.ChangesetClient;

	//TODO: Determine where things will be validated...too much going on already?
	rmps.core.util.ContextUtils = {

	/**
	 * Sets the context to the supplied context URI, and updates the recently viewed list.  Passing null
	 * for the contextURI parameter will set the context to the project context (root context.)
	 */
	setContext: function(contextURI, failHandler, expandedConfiguration){
		var context = jazz.app.context.get();
		// Otherwise set the context object properties
		context.baseline = contextURI;
		// update the parameters in the uri
		this._updateUrl(contextURI, null, expandedConfiguration);
	},
	
	/**
	 * Add context paramater(s) for a given url (an instance of string)	
	 */
	addContextParamaterToUrl: function(url, baseline) {
		// if there is no baseline then there is no changeset
		if(!baseline){
			return url;
		}
		
		// if the url is not a string we don't know how to append the context
		if(!dojo.isString(url)){
			return url;
		}
		
		
		var params = {};
		// we always add the baseline parameter
		params[IRmpsConstants.BASELINE] = baseline;
		
		return xhrUtil.addParamaterToUrl(url,params);
	},

	
	_updateUrl: function(contextURI, owningWorkspace, globalURI) {
		// Now fix the URL location
		var currentURL = location.search;
		var query = {};

		// Preserve any other parameters that are set
		if(currentURL && dojo.trim(currentURL) !== ""){
			var params = currentURL.substring(currentURL.indexOf("?") + 1, currentURL.length);
			query = dojo.queryToObject(params);
		}

		// Set the new context URI, generate the URL from the parameters we've been building, and set the URL
		if(contextURI){
			query[IRmpsConstants.BASELINE] = contextURI;
		}else{
			delete query[IRmpsConstants.BASELINE];
		}
		
		if (globalURI) {
			query.expandedConfiguration = globalURI;
		} else {
			delete query.expandedConfiguration;
		}
		var newURL = dojo.objectToQuery(query);

		// Update the recently viewed context list
		this._addContextToRecentlyViewed(contextURI);

		if(newURL.length > 0){
			
			// if switching context while editing a resource, we redirect to
			// the explorer because the resource may not be valid for the new
			// configuration
			var hash = window.location.hash;
			var editKey = "#action=rmps.app.design.edit&resource=";
			var createKey = "#action=rmps.app.design.create&resource=";
			var redirect = false;
			
			// validate whether the resource is exists in new context
			if (hash.indexOf(editKey) == 0) {
				var link = window.location.hash;
				if (link) {
					var params = dojo.queryToObject(link.substring(1));
					var resourceUri = params["resource"];
					if (resourceUri && this._isExistInConfiguration(resourceUri, contextURI) === false) {
						// redirect to explorer only if resource is not exists in new context
						redirect = true;
					}
				}
			} else if (hash.indexOf(createKey) == 0) {
				redirect = true;
			}
			if (redirect === true) {
				var link = window.location.href;
				var i = link.indexOf("?");
				if (i != -1) {
					window.location.href = link.substring(0, i+1) + newURL + "#action=rmps.app.design.explore";
					return;
				}
			}

			// remove the creating configuration tag
			var hash = window.location.hash;
			var pattern = "&creatingConfiguration=";
			var i = hash.indexOf(pattern);
			if (i != -1) {
				hash = hash.substring(0, i);
				var link = window.location.href;
				var index = link.indexOf("?");
				if (index != -1) {
					link = link.substring(0, index+1) + newURL + hash;
					window.location.href = link;
					return;
				}
			}
			
			if (window.location.search === "?" + newURL) {
				location.reload(true);
			}
			window.location.search = "?" + newURL;
		} else {
			window.location.search = "";
		}
	},
	
	_isExistInConfiguration: function(resourceUri, contextURI) {
		var params = {};
		params.resourceUri = resourceUri;
		params.configurationUri = contextURI;
		
		var link = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "queryvvc");
		link += "/getResourceVersion?" + dojo.objectToQuery(params);

		var args = {
			url: link,
			sync: true
		};
		var deferred =  jazz.client.xhrGet(args);
		var versionUri = deferred.ioArgs.xhr.getResponseHeader("Location");
		if (versionUri) {
			return true;
		}
		return false;
	},

	/**
	 * Helper function to get the URI of the current context.  This is useful when you're not sure the 
	 * true context.  The objected returned from jazz.app.context.get() contains a URI and a Baseline URI
	 * which is misleading.  The URI is actually the project and the Baseline URI, if set, is the URI of 
	 * the context. 
	 */
	getCurrentContextURI: function(){
		return jazz.app.context.get().baseline;
	},

	/**
	 * Helper function to get the URI of the current changeset if there is one. 
	 */
	getCurrentChangesetURI: function(){
		return jazz.app.context.get().changeset;
	},

	/**
	 * Looks for the current context, fetching it from the server if it has to.  Once the 
	 * context URI has been found, it will be passed into onFoundAction, which is expected to be a function
	 * On error (for example when project has no context when newly created), onError will be called.
	 * 
	 * projectId is optional, current project is default.
	 * 
	 * getCurrentContextURI, on the other hand is a wrapper for jazz.app.context.get().baseline
	 */
	findCurrentContextUriAndExecute: function(onFoundAction, projectId, onError){
		/*
		 * Searches, in order, for the current context:
		 * 1. Currently set context
		 * 2. Recently viewed context
		 * 3. Default stream context 
		 */
		dojo.when(this.getCurrentContextURI() || 
				this.getRecentlyViewedContextURIs()[0] ||
				StreamClient.getDefaultStream(projectId),
				// Pass results into onFoundAction and call it
				onFoundAction, onError);
	},

	/**
	 * Determines if a context is currently set by checking the baseline parameter.
	 */
	isContextSet: function(){
		return jazz.app.context.get().baseline ? true: false;
	}, 

	/**
	 * Checks if the context is valid.  If a context is supplied it will check that 
	 * context, otherwise the currently set context will be chosen.  Will execute
	 * the provided method if successful, and handle the error otherwise.  A special
	 * error handler can be provided, or if null then the default one will be used.
	 * 
	 * Note: This does not return a deferred object, rather it passes the supplied function
	 * to a deferred object.  If you would like the deferred object itself see the 
	 * `#getContextDeferred()` method.
	 * 
	 * Note: If you get frusterated passing variables into either of the supplied method remember
	 * the scope in which the method will be running.
	 */
	//TODO: add parameters for the supplied actions?
	validateContext: function(contextURI, validContextAction, invalidContextAction){
		// Set the context variable.  If the context is passed, use it otherwise get the 
		// context from the system.
		contextURI = contextURI ? contextURI : this.getCurrentContextURI();

		//deferred = rmps.core.baseline.BaselineClient.getBaseline(contextURI);
		// Make sure the proper service is used to handle the request.
		var deferred = this.getContextDeferred(contextURI);

		// Call the supplied validContextAction on success.  If the method doesnt exist then
		// don't do anything.
		if(validContextAction){
			deferred.addCallback(this, validContextAction);
		}

		/* 
		 * Handle an error.  If the user supplied an error handler, call that.  Otherwise call _defaultBadContextinvalidContextAction
		 *  with the provided error and add the deferred object as well.  In both cases, the invalid context will be removed from the
		 *  recently viewed context list
		 */ 

		deferred.addErrback(this, function(error){		
			if(invalidContextAction){
				invalidContextAction(deferred, error);
			} else {
				rmps.core.util.ContextUtils._defaultBadContextErrorHandler(deferred, error);
			}
		});

	}, // end validatecurrentContext

	/**
	 * Default error handler 
	 */
	_defaultBadContextErrorHandler: function(deferred, error){
		// Context isn't valid
		if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
			rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
		} else {
			rmps.core.error.createError(error).dispatch();
		}
	},

	/**
	 * Returns an array of URI's for the most recently viewed contexts.
	 */
	getRecentlyViewedContextURIs: function(){
		// TODO: Validate?  Would be nice to do in this function but how...?
		var cookie = dojo.cookie(COOKIE_NAME);

		// If the cookie doesnt exist, just return an empty array
		if(!cookie){
			return [];
		}

		// Split the list of URI's into an array
		var contextURIList = cookie.split(',', RECENTLY_VIEWED_LIMIT);

		// Blank entries indicate root workspace URI, fill them in.
		for(var index in contextURIList){
			if(contextURIList[index] === ""){
				contextURIList[index] = jazz.app.context.get().uri;
			}
		}

		// Return the array of contexts
		return contextURIList;
	},

	/**
	 * Returns the type of context (workspace, snapshot, stream, etc...) as defined in
	 *  rmps.core.util.ContextTypes
	 */
	getContextType: function(uri){
		var uriType;
		if(!uri){
			uri = this.getCurrentContextURI();
			if (!uri) {
				return null;
			}
		}

		var types = rmps.core.util.ContextTypes.values();
		var type;
		for(type in types){
			uriType = types[type];
			if(uri.indexOf(uriType.uriIdentifier) >= 0){
				return uriType;
			}
		}
		// default to a workspace
		return rmps.core.util.ContextTypes.WORKSPACE;
	},

	/**
	 * Convenience method to test for a workspace
	 */
	isWorkspace: function(uri){
		return this.getContextType(uri) === CONTEXT_TYPES.WORKSPACE;
	},

	/**
	 * Convenience method to test for a snapshot
	 */
	isSnapshot: function(uri){
		return this.getContextType(uri) === CONTEXT_TYPES.SNAPSHOT;
	},

	/**
	 * Returns the icon for the provided URI.  
	 */
	getIconUrlForContext: function(uri){
		return this.getContextType(uri).icon.uri;
	},
	
	getExpandedContext: function() {
		var search = window.location.search;
		if (search && search.length > 0) {
			var query = search.substring(1);
			var obj = dojo.queryToObject(query);
			
			return obj.expandedConfiguration;
		}
		
		return null;
	},

	/**
	 * Makes a request to the appropriate service for the provided URI, and
	 * returns the deferred object for use.
	 */
	getContextDeferred: function(uri, projectId){

		// Defaults to getting the current context
		if(!uri){
			uri = this.getCurrentContextURI();
		}

		var type = this.getContextType(uri);
		if(type === CONTEXT_TYPES.WORKSPACE){
			return rmps.core.streams.StreamClient.getStream(uri);
		} else {
			return rmps.core.baseline.BaselineClient.getBaseline(uri, projectId ? projectId : jazz.app.context.get().itemId);
		}
	},

	/**
	 * Adds the supplied context URI to the recently viewed store, automatically pushing out the last 
	 * context.
	 */
	_addContextToRecentlyViewed: function(contextURI){
		// Double check the context is set, if it's not then assume the project workspace
		if(!contextURI){
			contextURI = jazz.app.context.get().uri;
		}

		var cookie = dojo.cookie(COOKIE_NAME);
		var recent = this.getRecentlyViewedContextURIs();

		var buffer = [];

		// Iterate through the recent contexts to see if this context already exists in the array and simply needs to be moved to the top
		//TODO: Replace with removeContextFromRecentlyViewed
		for(var index = 0; index < recent.length; index++){
			// If the context isn't found add it to the buffer, if it is then skip it (it will be pushed to the start of the array later
			// Also note that you must explictly convert to strings!
			if(recent[index].toString() === contextURI.toString()){
				// stub
			}else{
				buffer.push(recent[index]);
			}
		}
		recent = buffer;

		// Remove the last element(s) if over the limit.  Make sure to leave room for the new context...!
		while(recent.length >= RECENTLY_VIEWED_LIMIT){
			recent.pop();
		}

		// Add the new element and set the cookie
		recent.unshift(contextURI);
		this._setRecentlyViewedCookieCookie(recent);
	},

	/**
	 * Removes the supplied context URI from the recently viewed store.
	 */
	removeContextFromRecentlyViewed: function(contextURI){
		// Note: validation not required
		var recent = this.getRecentlyViewedContextURIs();
		var buffer = [];

		// If the supplied URI is found in the recently viewed contexts list, exclude it when rebuilding
		// the list.
		var counter = 0;
		for(var index in recent){
			// Note: explicit conversion to strings for comparison
			if(recent[index] !== contextURI){
				buffer[counter++] = recent[index];
			}
		}		
		// Set the new cookie, expire after 30 days, valid for the current host and project
		this._setRecentlyViewedCookieCookie(buffer);
	},

	/**
	 * Sets the recently viewed cookie with the supplied data array 
	 */
	_setRecentlyViewedCookieCookie: function(data){

		// Expires in 90 days
		var expires = 90;

		// Set the domain 
		var domain = window.location.hostname;

		// Make the cookie project dependent
		var path = window.location.pathname;

		dojo.cookie(COOKIE_NAME, data, {
			"path": path,
			"domain" : domain,
			"expires" : expires
		});
	},
	
	/**
	 * Returns the URI that should be set in the current window to switch to the given
	 * context
	 */
	getContextUri: function(contextUri){
		return this.getQueryForContext(contextUri) + location.hash;
	},

	
	/**
	 * Returns the current context parameters. Including the changeset if there is a changeset
	 */
	computeContextParams: function(shorten){
		var contextParam;
		var baseline = jazz.app.context.get().baseline;
		
		if (shorten) {
			var index = baseline.indexOf(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL);
			if (index !== -1) {
				baseline = baseline.substring(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL.length + 1);
			}
		}
		
		if (baseline) {
			contextParam = IRmpsConstants.OSLC_CONFIG + "=" + encodeURIComponent(baseline);
		}		
		return contextParam;
	},
	
	/*
	 * Returns the query portion of the URL that would switch to the given context.  
	 * if the contextUri is null the current context is used.
	 */
	getQueryForContext: function(contextUri) {
		var queryWithContext= "";
		var currentSearch = location.search;
		var queryObject = null;
		
		if (currentSearch && dojo.trim(currentSearch) != "") {
			//there are current parameter; don't overwrite the others
			 var query = currentSearch.substring(currentSearch.indexOf("?") + 1, currentSearch.length);
			queryObject = dojo.queryToObject(query);
		} else {
			queryObject = {};
		}
		
		if (contextUri) {
			queryObject[IRmpsConstants.BASELINE] = contextUri;
		} else {
			queryObject[IRmpsConstants.BASELINE] = jazz.app.context.get().baseline;
		}
		
		var newSearch = dojo.objectToQuery(queryObject);
		if (newSearch.length > 0) {
			queryWithContext = "?" + newSearch;
		} else {
			queryWithContext = "";
		}
		return queryWithContext;
	},
	
	getConfigureContextURL: function() {
		return PROJECT_AREA_PATH + jazz.app.context.get().itemId;
	},
	
	getProjectSpace: function(projectId) {
		if (!projectId) {
			projectId = jazz.app.context.get().itemId;
		}
		var args = {
			url: net.jazz.ajax.contextRoot() + "/queryvvc/spacefromcontext?contextUri=" + escape(projectId),
			headers: {
				"Accept" : "application/json"
			},
			handleAs: "json",
			sync: true,
			load: function(response, ioArgs) {
				return ioArgs.xhr.getResponseHeader("Location");
			}
		};
		return jazz.client.xhrGet(args);
	},
	
	getContextUriFromVvcContextUri: function(vvcContextUri, projectId) {
		if (vvcContextUri) {
			var link = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "queryvvc");
			if (!projectId) {
				projectId = jazz.app.context.get().itemId;
			}
			link += "/contextfromconfig?configurationUri="
					+ escape(vvcContextUri)
					+ "&projectId=" + projectId;

			var args = {
				url: link,
				sync: true
			};
			var deferred =  jazz.client.xhrGet(args);
			return deferred.ioArgs.xhr.getResponseHeader("Location");
		}
	},
	
	getDMConfigurationFromVVCConfiguration: function(vvcConfigurationUri) {
		var str = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL;
		if (vvcConfigurationUri.slice(0, str.length) === str) {
			// Already a DM configuration
			return vvcConfigurationUri;
		}

		var ix = vvcConfigurationUri.indexOf('/configurations/');
		if (ix === -1) {
			// Probably a GC
			return vvcConfigurationUri;
		}
		var vvcPrefix = vvcConfigurationUri.substring(0, ix);
		
		return vvcConfigurationUri.replace(vvcPrefix, net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + '/oslc_config');
	},
	
	getVVCConfigurationFromDMConfiguration: function(dmConfigurationUri, vvcUrl) {
		var str = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL;
		if (dmConfigurationUri.slice(0, str.length) !== str) {
			return dmConfigurationUri;
		}
		var ix = dmConfigurationUri.indexOf('/configurations/');
		if (ix === -1) {
			// Probably a GC
			return dmConfigurationUri;
		}
		var dmPrefix = dmConfigurationUri.substring(0, ix);

		return dmConfigurationUri.replace(dmPrefix, vvcUrl);
	},
	
	getLCfromGCandComponent: function(gcUri, componentUri, lcNotFoundFn) {
		if (!lcNotFoundFn) {
			lcNotFoundFn = this._handleLCNotFound;						
		}
		
		var def = new dojo.Deferred();
		this._getLCfromGCandComponent(gcUri, componentUri, dojo.hitch(this, function(gcUri, componentUri) {
			// We got a 404, for backward compatibility also check with old component URI format
			var legacyComponentUri = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/oslc_config/components/" + jazz.app.context.get().itemId;
			this._getLCfromGCandComponent(gcUri, componentUri, lcNotFoundFn, legacyComponentUri).then(function(lc) {
				def.callback(lc);
			});
		})).then(function(lc) {
			def.callback(lc);
		});
		return def;
	},
	
	_getLCfromGCandComponent: function(gcUri, componentUri, lcNotFoundFn, legacyComponentUri) {
		var ix = gcUri.lastIndexOf('/');
		var gcConfigurationService = gcUri.substring(0, ix);

		// TODO: LC selection? Service returns first match. findAll param can be added to get all
		var link = gcConfigurationService + "?contextConfigurationUri="
		 + escape(gcUri) + "&componentUri=" + escape(legacyComponentUri || componentUri);
		
		var args = {
			headers: {
				'Accept': 'application/x-www-form-urlencoded'
			},
			url: link,
			sync: true,
			load: function(response, ioArgs) {
				return response.split('=')[1].split(',');
			},
			error: dojo.hitch(this, function(response, ioArgs) {
				if (ioArgs.xhr.status === 404) {
					lcNotFoundFn(gcUri, componentUri);
				} else {
					var error = rmps.core.error.createXhrError(ioArgs);
					error.dispatch();					
				}
			})
		};
		return jazz.client.xhrGet(args);
	},

	_handleLCNotFound: function(gcUri, componentUri) {
		// Get name of GC and component (project area)
		var args1 = {
			headers: {
				'Accept': 'application/json'
			},
			url: gcUri
		};
		var def1 = jazz.client.xhrGet(args1);

		var args2 = {
			headers: {
				'Accept': 'application/json'
			},
			url: componentUri	
		};
		var def2 = jazz.client.xhrGet(args2);
		
		var defList = new dojo.DeferredList([def1, def2]);
		defList.then(function(result) {
			var gcName = result[0][0] ? dojo.fromJson(result[0][1])[gcUri]["http://purl.org/dc/terms/title"][0].value : gcUri;
			var compName = result[1][0] ? dojo.fromJson(result[1][1])[componentUri]["http://purl.org/dc/terms/title"][0].value : componentUri;
			ContextDisplayMessagesUtil.displayInvalidContextForProjectArea(compName, gcName);
		});
	}


	}; // end of ContextUtil

}());